@@ -2,7 +2,7 @@
 
     File: adv.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -147,7 +147,8 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
       aff_copy(stdscr);
       wmove(stdscr,4,0);
       wdoprintf(stdscr,"%s",disk_car->description(disk_car));
-      mvwaddstr(stdscr,6,0,msg_PART_HEADER_LONG);
+      if(list_part!=NULL)
+	mvwaddstr(stdscr,6,0,msg_PART_HEADER_LONG);
       rewrite=0;
     }
     for(i=0,element=list_part;(element!=NULL) && (i<offset);element=element->next,i++);
@@ -165,9 +166,16 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 	aff_part(stdscr,AFF_PART_ORDER,disk_car,element->part);
       }
     }
-    options="tq";
     menu=0;
-    if(current_element!=NULL)
+    if(current_element==NULL)
+    {
+      options="q";
+      wmove(stdscr,5+2,0);
+      wattrset(stdscr,A_STANDOUT);
+      wdoprintf(stdscr,"No partition avaible.");
+      wattroff(stdscr,A_STANDOUT);
+    }
+    else
     {
       if(is_fat(current_element->part) ||
 	  is_ntfs(current_element->part))
@@ -187,12 +195,14 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 	menuAdv[2].desc="Locate HFS+ backup superblock";
 	menu=1;
       }
+      else
+	options="tq";
     }
     quit=0;
     if(*current_cmd!=NULL)
     {
       int keep_asking;
-      command='Q';
+      command='q';
       do
       {
 	keep_asking=0;
@@ -314,7 +324,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 
 int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
-  unsigned char buffer_bs[0x200];
+  unsigned char *buffer_bs;
   const char *options="DR";
   int rescan=1;
   struct MenuItem menu_fat1x[]=
@@ -328,6 +338,7 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
     { 'I', "Init Root","Init root directory: Very Dangerous! Expert only"},
     { 0, NULL, NULL }
   };
+  buffer_bs=(unsigned char*)MALLOC(FAT1x_BOOT_SECTOR_SIZE);
   while(1)
   {
     int command;
@@ -347,10 +358,10 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       else
 	options="DRCI";
       aff_buffer(BUFFER_ADD,"Boot sector\n");
-      if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset)!=0)
+      if(disk_car->read(disk_car,FAT1x_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset)!=0)
       {
 	aff_buffer(BUFFER_ADD,"fat1x_boot_sector: Can't read boot sector.\n");
-	memset(&buffer_bs,0,sizeof(buffer_bs));
+	memset(buffer_bs,0,FAT1x_BOOT_SECTOR_SIZE);
       }
       if(test_FAT(disk_car,(const struct fat_boot_sector *)buffer_bs,partition,debug,0)==0)
       {
@@ -395,6 +406,7 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
     switch(command)
     {
       case 0:
+	free(buffer_bs);
 	return 0;
       case 'R': /* R : rebuild boot sector */
 	rebuild_FAT_BS(disk_car,partition,debug,dump_ind,1,expert,current_cmd);
@@ -411,7 +423,7 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
 	  ecrit_rapport("Boot sector\n");
 	  mvwaddstr(window,6,0, "Boot sector");
-	  dump(window,buffer_bs,DEFAULT_SECTOR_SIZE);
+	  dump(window,buffer_bs,FAT1x_BOOT_SECTOR_SIZE);
 	  delwin(window);
 	  (void) clearok(stdscr, TRUE);
 #ifdef HAVE_TOUCHWIN
@@ -431,8 +443,8 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 
 int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
-  unsigned char buffer_bs[3*disk_car->sector_size];
-  unsigned char buffer_backup_bs[3*disk_car->sector_size];
+  unsigned char *buffer_bs;
+  unsigned char *buffer_backup_bs;
   const char *options="DRC";
   int rescan=1;
   struct MenuItem menu_fat32[]=
@@ -447,7 +459,8 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
     { 'C', "Repair FAT","Very Dangerous! Expert only"},
     { 0, NULL, NULL }
   };
-
+  buffer_bs=(unsigned char*)MALLOC(3*disk_car->sector_size);
+  buffer_backup_bs=(unsigned char*)MALLOC(3*disk_car->sector_size);
   while(1)
   {
     int command;
@@ -467,10 +480,10 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       ecrit_rapport("\nfat32_boot_sector\n");
       aff_part_rapport(disk_car,partition);
       aff_buffer(BUFFER_ADD,"Boot sector\n");
-      if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset)!=0)
+      if(disk_car->read(disk_car,3*disk_car->sector_size, buffer_bs, partition->part_offset)!=0)
       {
 	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read boot sector.\n");
-	memset(&buffer_bs,0,sizeof(buffer_bs));
+	memset(buffer_bs,0,3*disk_car->sector_size);
       }
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_bs,partition,debug,0)==0)
       {
@@ -486,10 +499,10 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	}
       }
       aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
-      if(disk_car->read(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+6*disk_car->sector_size)!=0)
+      if(disk_car->read(disk_car,3*disk_car->sector_size, buffer_backup_bs, partition->part_offset+6*disk_car->sector_size)!=0)
       {
 	aff_buffer(BUFFER_ADD,"fat32_boot_sector: Can't read backup boot sector.\n");
-	memset(&buffer_backup_bs,0,sizeof(buffer_backup_bs));
+	memset(buffer_backup_bs,0,3*disk_car->sector_size);
       }
       if(test_FAT(disk_car,(struct fat_boot_sector *)buffer_backup_bs,partition,debug,0)==0)
       {
@@ -505,19 +518,19 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	}
       }
       aff_buffer(BUFFER_ADD,"\n");
-      if((memcmp(buffer_bs,buffer_backup_bs,0x3E8)==0)&&(memcmp(&buffer_bs[0x3F0],&buffer_backup_bs[0x3F0],0x600-0x3F0))==0)
+      if((memcmp(buffer_bs,buffer_backup_bs,0x3E8)==0)&&(memcmp(buffer_bs+0x3F0,buffer_backup_bs+0x3F0,0x600-0x3F0))==0)
       {
 	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
 	opt_over=0;
       }
       else
       {
-	if(memcmp(buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE)!=0)
+	if(memcmp(buffer_bs,buffer_backup_bs,0x200)!=0)
 	  aff_buffer(BUFFER_ADD,"First sectors (Boot code and partition information) are not identical.\n");
-	if((memcmp(&buffer_bs[disk_car->sector_size],&buffer_backup_bs[disk_car->sector_size],0x1E8)!=0)||
-	    (memcmp(&buffer_bs[disk_car->sector_size+0x1F0],&buffer_backup_bs[disk_car->sector_size+0x1F0],0x200-0x1F0)!=0))
+	if((memcmp(buffer_bs+disk_car->sector_size, buffer_backup_bs+disk_car->sector_size,0x1E8)!=0)||
+	    (memcmp(buffer_bs+disk_car->sector_size+0x1F0, buffer_backup_bs+disk_car->sector_size+0x1F0,0x200-0x1F0)!=0))
 	  aff_buffer(BUFFER_ADD,"Second sectors (cluster information) are not identical.\n");
-	if(memcmp(&buffer_bs[2*disk_car->sector_size],&buffer_backup_bs[2*disk_car->sector_size],DEFAULT_SECTOR_SIZE)!=0)
+	if(memcmp(buffer_bs+2*disk_car->sector_size, buffer_backup_bs+2*disk_car->sector_size,0x200)!=0)
 	  aff_buffer(BUFFER_ADD,"Third sectors (Second part of boot code) are not identical.\n");
       }
       aff_buffer(BUFFER_ADD,"\n");
@@ -574,12 +587,14 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
     switch(command)
     {
       case 0:
+	free(buffer_bs);
+	free(buffer_backup_bs);
 	return 0;
       case 'O': /* O : copy original boot sector over backup boot */
 	if(ask_confirmation("Copy original FAT32 boot sector over backup boot, confirm ? (Y/N)")!=0)
 	{
 	  ecrit_rapport("copy original boot sector over backup boot\n");
-	  if(disk_car->write(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+6*disk_car->sector_size)!=0)
+	  if(disk_car->write(disk_car,3*disk_car->sector_size, buffer_bs, partition->part_offset+6*disk_car->sector_size)!=0)
 	  {
 	    display_message("Write error: Can't overwrite FAT32 backup boot sector\n");
 	  }
@@ -590,7 +605,7 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	if(ask_confirmation("Copy backup FAT32 boot sector over boot sector, confirm ? (Y/N)")!=0)
 	{
 	  ecrit_rapport("copy backup boot sector over boot sector\n");
-	  if(disk_car->write(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset)!=0)
+	  if(disk_car->write(disk_car,3*disk_car->sector_size, buffer_backup_bs, partition->part_offset)!=0)
 	  {
 	    display_message("Write error: Can't overwrite FAT32 boot sector\n");
 	  }
@@ -618,7 +633,7 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 #ifdef HAVE_TOUCHWIN
 	  touchwin(stdscr);
 #endif
-	  dump_2fat_rapport((const struct fat_boot_sector*)&buffer_bs,(const struct fat_boot_sector*)&buffer_backup_bs,UP_FAT32,disk_car->sector_size);
+	  dump_2fat_rapport((const struct fat_boot_sector*)buffer_bs,(const struct fat_boot_sector*)buffer_backup_bs,UP_FAT32,disk_car->sector_size);
 	}
 	break;
       case 'C':
@@ -630,8 +645,8 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 
 int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
-  unsigned char buffer_bs[DEFAULT_SECTOR_SIZE];
-  unsigned char buffer_backup_bs[DEFAULT_SECTOR_SIZE];
+  unsigned char *buffer_bs;
+  unsigned char *buffer_backup_bs;
   const char *options="";
   int rescan=1;
   struct MenuItem menu_ntfs[]=
@@ -646,6 +661,8 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
     { 'D', "Dump","Dump boot sector and backup boot sector"},
     { 0, NULL, NULL }
   };
+  buffer_bs=(unsigned char*)MALLOC(NTFS_BOOT_SECTOR_SIZE);
+  buffer_backup_bs=(unsigned char*)MALLOC(NTFS_BOOT_SECTOR_SIZE);
 
   while(1)
   {
@@ -666,39 +683,39 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
       ecrit_rapport("\nntfs_boot_sector\n");
       aff_part_rapport(disk_car,partition);
       aff_buffer(BUFFER_ADD,"Boot sector\n");
-      if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset)!=0)
+      if(disk_car->read(disk_car,NTFS_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset)!=0)
       {
 	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read boot sector.\n");
-	memset(&buffer_bs,0,sizeof(buffer_bs));
+	memset(buffer_bs,0,NTFS_BOOT_SECTOR_SIZE);
       }
       if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_bs,partition,debug,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"OK\n");
+	aff_buffer(BUFFER_ADD,"Status: OK\n");
 	opt_O=1;
 	opt_over=1;
       }
       else
       {
-	aff_buffer(BUFFER_ADD,"Bad\n");
+	aff_buffer(BUFFER_ADD,"Status: Bad\n");
       }
       aff_buffer(BUFFER_ADD,"\nBackup boot sector\n");
-      if(disk_car->read(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
+      if(disk_car->read(disk_car,NTFS_BOOT_SECTOR_SIZE, buffer_backup_bs, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
       {
 	aff_buffer(BUFFER_ADD,"ntfs_boot_sector: Can't read backup boot sector.\n");
-	memset(&buffer_backup_bs,0,sizeof(buffer_backup_bs));
+	memset(buffer_backup_bs,0,NTFS_BOOT_SECTOR_SIZE);
       }
       if(test_NTFS(disk_car,(struct ntfs_boot_sector*)buffer_backup_bs,partition,debug,0)==0)
       {
-	aff_buffer(BUFFER_ADD,"OK\n");
+	aff_buffer(BUFFER_ADD,"Status: OK\n");
 	opt_B=1;
 	opt_over=1;
       }
       else
       {
-	aff_buffer(BUFFER_ADD,"Bad\n");
+	aff_buffer(BUFFER_ADD,"Status: Bad\n");
       }
       aff_buffer(BUFFER_ADD,"\n");
-      if(memcmp(buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE)==0)
+      if(memcmp(buffer_bs,buffer_backup_bs,NTFS_BOOT_SECTOR_SIZE)==0)
       {
 	dump_ntfs_rapport((const struct ntfs_boot_sector *)buffer_bs);
 	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
@@ -757,12 +774,14 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
     switch(command)
     {
       case 0:
+        free(buffer_bs);
+        free(buffer_backup_bs);
 	return 0;
       case 'O': /* O : copy original boot sector over backup boot */
 	if(ask_confirmation("Copy original NTFS boot sector over backup boot, confirm ? (Y/N)")!=0)
 	{
 	  ecrit_rapport("copy original boot sector over backup boot\n");
-	  if(disk_car->write(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
+	  if(disk_car->write(disk_car,NTFS_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
 	  {
 	    display_message("Write error: Can't overwrite NTFS backup boot sector\n");
 	  }
@@ -773,7 +792,7 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	if(ask_confirmation("Copy backup NTFS boot sector over boot sector, confirm ? (Y/N)")!=0)
 	{
 	  ecrit_rapport("copy backup boot sector over boot sector\n");
-	  if(disk_car->write(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset)!=0)
+	  if(disk_car->write(disk_car,NTFS_BOOT_SECTOR_SIZE, buffer_backup_bs, partition->part_offset)!=0)
 	  {
 	    display_message("Write error: Can't overwrite NTFS boot sector\n");
 	  }
@@ -798,7 +817,7 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
 	  ecrit_rapport("Boot sector                        Backup boot sector\n");
 	  mvwaddstr(window,6,0, "Boot sector                        Backup boot sector");
-	  dump2(window,buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE);
+	  dump2(window,buffer_bs,buffer_backup_bs,NTFS_BOOT_SECTOR_SIZE);
 	  delwin(window);
 	  (void) clearok(stdscr, TRUE);
 #ifdef HAVE_TOUCHWIN
@@ -812,8 +831,8 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 
 int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
-  unsigned char buffer_bs[0x400];
-  unsigned char buffer_backup_bs[0x400];
+  unsigned char *buffer_bs;
+  unsigned char *buffer_backup_bs;
   const char *options="";
   int rescan=1;
   struct MenuItem menu_hfsp[]=
@@ -826,6 +845,8 @@ int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
     { 'D', "Dump","Dump superblock and backup superblock"},
     { 0, NULL, NULL }
   };
+  buffer_bs=(unsigned char*)MALLOC(HFSP_BOOT_SECTOR_SIZE);
+  buffer_backup_bs=(unsigned char*)MALLOC(HFSP_BOOT_SECTOR_SIZE);
 
   while(1)
   {
@@ -846,10 +867,10 @@ int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
       ecrit_rapport("\nHFSP_boot_sector\n");
       aff_part_rapport(disk_car,partition);
       aff_buffer(BUFFER_ADD,"Superblock\n");
-      if(disk_car->read(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+0x400)!=0)
+      if(disk_car->read(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset+0x400)!=0)
       {
 	aff_buffer(BUFFER_ADD,"HFSP_boot_sector: Can't read superblock.\n");
-	memset(&buffer_bs,0,sizeof(buffer_bs));
+	memset(buffer_bs,0,HFSP_BOOT_SECTOR_SIZE);
       }
       if(test_HFSP(disk_car,(struct hfsp_vh*)buffer_bs,partition,debug,0)==0)
       {
@@ -862,10 +883,10 @@ int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	aff_buffer(BUFFER_ADD,"Bad\n");
       }
       aff_buffer(BUFFER_ADD,"\nBackup superblock\n");
-      if(disk_car->read(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+partition->part_size-0x400)!=0)
+      if(disk_car->read(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_backup_bs, partition->part_offset+partition->part_size-0x400)!=0)
       {
 	aff_buffer(BUFFER_ADD,"HFSP_boot_sector: Can't read backup superblock.\n");
-	memset(&buffer_backup_bs,0,sizeof(buffer_backup_bs));
+	memset(buffer_backup_bs,0,HFSP_BOOT_SECTOR_SIZE);
       }
       if(test_HFSP(disk_car,(struct hfsp_vh*)buffer_backup_bs,partition,debug,0)==0)
       {
@@ -878,7 +899,7 @@ int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	aff_buffer(BUFFER_ADD,"Bad\n");
       }
       aff_buffer(BUFFER_ADD,"\n");
-      if(memcmp(buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE)==0)
+      if(memcmp(buffer_bs,buffer_backup_bs,HFSP_BOOT_SECTOR_SIZE)==0)
       {
 	aff_buffer(BUFFER_ADD,"Sectors are identical.\n");
 	opt_over=0;
@@ -927,12 +948,14 @@ int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
     switch(command)
     {
       case 0:
+        free(buffer_bs);
+        free(buffer_backup_bs);
 	return 0;
       case 'O': /* O : copy original superblock over backup boot */
 	if(ask_confirmation("Copy original HFSP superblock over backup boot, confirm ? (Y/N)")!=0)
 	{
 	  ecrit_rapport("copy original superblock over backup boot\n");
-	  if(disk_car->write(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+partition->part_size-0x400)!=0)
+	  if(disk_car->write(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_bs, partition->part_offset+partition->part_size-0x400)!=0)
 	  {
 	    display_message("Write error: Can't overwrite HFSP backup superblock\n");
 	  }
@@ -943,7 +966,7 @@ int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	if(ask_confirmation("Copy backup HFSP superblock over main superblock, confirm ? (Y/N)")!=0)
 	{
 	  ecrit_rapport("copy backup superblock over main superblock\n");
-	  if(disk_car->write(disk_car,sizeof(buffer_backup_bs), &buffer_backup_bs, partition->part_offset+0x400)!=0)
+	  if(disk_car->write(disk_car,HFSP_BOOT_SECTOR_SIZE, buffer_backup_bs, partition->part_offset+0x400)!=0)
 	  {
 	    display_message("Write error: Can't overwrite HFSP main superblock\n");
 	  }
@@ -961,7 +984,7 @@ int HFSP_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	  aff_part(window,AFF_PART_ORDER,disk_car,partition);
 	  ecrit_rapport("Superblock                        Backup superblock\n");
 	  mvwaddstr(window,6,0, "Superblock                        Backup superblock");
-	  dump2(window,buffer_bs,buffer_backup_bs,DEFAULT_SECTOR_SIZE);
+	  dump2(window,buffer_bs,buffer_backup_bs,HFSP_BOOT_SECTOR_SIZE);
 	  delwin(window);
 	  (void) clearok(stdscr, TRUE);
 #ifdef HAVE_TOUCHWIN
