@@ -2,7 +2,7 @@
 
     File: analyse.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,17 +15,20 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
- 
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+//#include <assert.h>
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -54,13 +57,13 @@
 #include "ufs.h"
 #include "xfs.h"
 
-int search_NTFS_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
+int search_NTFS_backup(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
 {
-  unsigned char buffer[DEFAULT_SECTOR_SIZE];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+//  assert(sizeof(struct ntfs_boot_sector)<=DEFAULT_SECTOR_SIZE);
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
     return -1;
   /* NTFS recovery using backup sector */
-  if(recover_NTFS(disk_car,(struct ntfs_boot_sector*)&buffer,partition,debug,dump_ind,1)==0)
+  if(recover_NTFS(disk_car,(struct ntfs_boot_sector*)buffer,partition,debug,dump_ind,1)==0)
   {
     strncpy(partition->info,"NTFS found using backup sector!",sizeof(partition->info));
     return 1;
@@ -68,18 +71,19 @@ int search_NTFS_backup(t_param_disk *disk_car,t_partition *partition, const int
   return 0;
 }
 
-int search_HFS_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
+int search_HFS_backup(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
 {
-  unsigned char buffer[0x400];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+//  assert(sizeof(hfs_mdb_t)<=0x400);
+//  assert(sizeof(struct hfsp_vh)==0x200);
+  if(disk_car->read(disk_car,0x400, buffer, partition->part_offset)!=0)
     return -1;
   /* HFS recovery using backup sector */
-  if(recover_HFS(disk_car,(const hfs_mdb_t*)&buffer,partition,debug,dump_ind,1)==0)
+  if(recover_HFS(disk_car,(const hfs_mdb_t*)buffer,partition,debug,dump_ind,1)==0)
   {
     strncpy(partition->info,"HFS found using backup sector!",sizeof(partition->info));
     return 1;
   }
-  if(recover_HFSP(disk_car,(const struct hfsp_vh*)&buffer,partition,debug,dump_ind,1)==0)
+  if(recover_HFSP(disk_car,(const struct hfsp_vh*)buffer,partition,debug,dump_ind,1)==0)
   {
     strncpy(partition->info,"HFS+ found using backup sector!",sizeof(partition->info));
     return 1;
@@ -87,13 +91,13 @@ int search_HFS_backup(t_param_disk *disk_car,t_partition *partition, const int d
   return 0;
 }
 
-int search_FAT_backup(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
+int search_FAT_backup(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+//  assert(sizeof(struct fat_boot_sector)==DEFAULT_SECTOR_SIZE);
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
     return -1;
   /* FAT32 recovery using backup sector */
-  if(recover_FAT(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind,1)==0)
+  if(recover_FAT(disk_car,(const struct fat_boot_sector*)buffer,partition,debug,dump_ind,1)==0)
   {
     strncpy(partition->info,"FAT found using backup sector!",sizeof(partition->info));
     return 1;
@@ -101,102 +105,116 @@ int search_FAT_backup(t_param_disk *disk_car,t_partition *partition, const int d
   return 0;
 }
 
-int search_type_0(t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
+int search_type_0(unsigned char *buffer,t_param_disk *disk_car,t_partition *partition, const int debug, const int dump_ind)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+//  assert(sizeof(union swap_header)<=8*DEFAULT_SECTOR_SIZE);
+//  assert(sizeof(pv_disk_t)<=8*DEFAULT_SECTOR_SIZE);
+//  assert(sizeof(struct fat_boot_sector)<=8*DEFAULT_SECTOR_SIZE);
+//  assert(sizeof(struct ntfs_boot_sector)<=8*DEFAULT_SECTOR_SIZE);
+//  assert(sizeof(struct disk_netware)<=8*DEFAULT_SECTOR_SIZE);
+//  assert(sizeof(struct xfs_sb)<=8*DEFAULT_SECTOR_SIZE);
+//  assert(sizeof(struct disk_fatx)<=8*DEFAULT_SECTOR_SIZE);
   if(debug>2)
   {
     ecrit_rapport("search_type_0 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  if(disk_car->read(disk_car,8*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
     return -1;
-  if(recover_Linux_SWAP(disk_car,(const union swap_header *)&buffer,partition,debug,dump_ind)==0) return 1;
-  if(recover_LVM(disk_car,(const pv_disk_t*)&buffer,partition,debug,dump_ind)==0) return 1;
-  if(recover_FAT(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind,0)==0) return 1;
-  if(recover_HPFS(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind)==0) return 1;
-  if(recover_OS2MB(disk_car,(const struct fat_boot_sector*)&buffer,partition,debug,dump_ind)==0) return 1;
-  if(recover_NTFS(disk_car,(const struct ntfs_boot_sector*)&buffer,partition,debug,dump_ind,0)==0) return 1;
-  if(recover_netware(disk_car,(const struct disk_netware *)&buffer,partition)==0) return 1;
-  if(recover_xfs(disk_car,(const struct xfs_sb*)&buffer,partition,debug,dump_ind)==0) return 1;
-  if(recover_FATX(disk_car,(const struct disk_fatx*)&buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_Linux_SWAP(disk_car,(const union swap_header *)buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_LVM(disk_car,(const pv_disk_t*)buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_FAT(disk_car,(const struct fat_boot_sector*)buffer,partition,debug,dump_ind,0)==0) return 1;
+  if(recover_HPFS(disk_car,(const struct fat_boot_sector*)buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_OS2MB(disk_car,(const struct fat_boot_sector*)buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_NTFS(disk_car,(const struct ntfs_boot_sector*)buffer,partition,debug,dump_ind,0)==0) return 1;
+  if(recover_netware(disk_car,(const struct disk_netware *)buffer,partition)==0) return 1;
+  if(recover_xfs(disk_car,(const struct xfs_sb*)buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_FATX(disk_car,(const struct disk_fatx*)buffer,partition,debug,dump_ind)==0) return 1;
   return 0;
 }
 
-int search_type_1(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
+int search_type_1(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+//  assert(sizeof(struct disklabel)<=2*0x200);
+//  assert(sizeof(struct disk_super_block)<=0x200);
+//  assert(sizeof(struct cramfs_super)<=2*0x200);
+//  assert(sizeof(struct sysv4_super_block)<=2*0x200);
+//  assert(sizeof(sun_partition_i386)<=2*0x200);
   if(debug>2)
   {
     ecrit_rapport("search_type_1 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  if(disk_car->read(disk_car,8*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
     return -1;
-  if(recover_BSD(disk_car,(const struct disklabel *)&buffer[0x200],partition,debug,dump_ind)==0) return 1;
-  if(recover_BeFS(disk_car,(const struct disk_super_block *)&buffer[0x200],partition,debug,dump_ind)==0) return 1;
-  if(recover_cramfs(disk_car,(const struct cramfs_super*)&buffer[0x200],partition,debug,dump_ind)==0) return 1;
-  if(recover_sysv(disk_car,&buffer[0x200],partition,debug,dump_ind)==0) return 1;
-  if(recover_LVM2(disk_car,(const unsigned char*)&buffer[0x200],partition,debug,dump_ind)==0) return 1;
-  if(recover_sun_i386(disk_car,(const sun_partition_i386*)&buffer[0x200],partition,debug,dump_ind)==0) return 1;
+  if(recover_BSD(disk_car,(const struct disklabel *)(buffer+0x200),partition,debug,dump_ind)==0) return 1;
+  if(recover_BeFS(disk_car,(const struct disk_super_block *)(buffer+0x200),partition,debug,dump_ind)==0) return 1;
+  if(recover_cramfs(disk_car,(const struct cramfs_super*)(buffer+0x200),partition,debug,dump_ind)==0) return 1;
+  if(recover_sysv(disk_car,(const struct sysv4_super_block*)(buffer+0x200),partition,debug,dump_ind)==0) return 1;
+  if(recover_LVM2(disk_car,(const unsigned char*)(buffer+0x200),partition,debug,dump_ind)==0) return 1;
+  if(recover_sun_i386(disk_car,(const sun_partition_i386*)(buffer+0x200),partition,debug,dump_ind)==0) return 1;
   return 0;
 }
 
-int search_type_2(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
+int search_type_2(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+//  assert(sizeof(struct ext2_super_block)<=1024);
+//  assert(sizeof(hfs_mdb_t)<=1024);
+//  assert(sizeof(struct hfsp_vh)<=1024);
   if(debug>2)
   {
     ecrit_rapport("search_type_2 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,1024, &buffer[0x400], partition->part_offset+1024)!=0)
+  if(disk_car->read(disk_car,1024, (buffer+0x400), partition->part_offset+1024)!=0)
     return -1;
-  if(recover_EXT2(disk_car,(const struct ext2_super_block*)&buffer[0x400],partition,debug,dump_ind)==0) return 1;
-  if(recover_HFS(disk_car,(const hfs_mdb_t*)&buffer[0x400],partition,debug,dump_ind,0)==0) return 1;
-  if(recover_HFSP(disk_car,(const struct hfsp_vh*)&buffer[0x400],partition,debug,dump_ind,0)==0) return 1;
+  if(recover_EXT2(disk_car,(const struct ext2_super_block*)(buffer+0x400),partition,debug,dump_ind)==0) return 1;
+  if(recover_HFS(disk_car,(const hfs_mdb_t*)(buffer+0x400),partition,debug,dump_ind,0)==0) return 1;
+  if(recover_HFSP(disk_car,(const struct hfsp_vh*)(buffer+0x400),partition,debug,dump_ind,0)==0) return 1;
   return 0;
 }
 
-int search_type_16(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
+int search_type_16(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+//  assert(sizeof(struct ufs_super_block)<=3*DEFAULT_SECTOR_SIZE);
   if(debug>2)
   {
     ecrit_rapport("search_type_16 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+16*512)!=0) /* 8k offset */
+  if(disk_car->read(disk_car,3*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+16*512)!=0) /* 8k offset */
     return -1;
   /* Test UFS */
-  if(recover_ufs(disk_car,(const struct ufs_super_block*)&buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_ufs(disk_car,(const struct ufs_super_block*)buffer,partition,debug,dump_ind)==0) return 1;
   return 0;
 }
 
-int search_type_64(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
+int search_type_64(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+//  assert(sizeof(struct jfs_superblock)<=2*DEFAULT_SECTOR_SIZE);
   if(debug>2)
   {
     ecrit_rapport("search_type_64 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
   /* Test JFS */
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+64*512)!=0) /* 32k offset */
+  if(disk_car->read(disk_car,2*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+64*512)!=0) /* 32k offset */
     return -1;
-  if(recover_JFS(disk_car,(const struct jfs_superblock*)&buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_JFS(disk_car,(const struct jfs_superblock*)buffer,partition,debug,dump_ind)==0) return 1;
   return 0;
 }
 
-int search_type_128(t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
+int search_type_128(unsigned char *buffer, t_param_disk *disk_car,t_partition *partition,const int debug, const int dump_ind)
 {
-  unsigned char buffer[16*DEFAULT_SECTOR_SIZE];
-  /* Reiserfs4 need to read the master superblock and the format40 superblock => 2*4096 */
+  /* Reiserfs4 need to read the master superblock and the format40 superblock => 4096 */
+//  assert(sizeof(struct reiserfs_super_block)<=9*DEFAULT_SECTOR_SIZE);
+//  assert(4096+sizeof(struct format40_super)<=9*DEFAULT_SECTOR_SIZE);
+//  assert(sizeof(struct ufs_super_block)<=9*DEFAULT_SECTOR_SIZE);
   if(debug>2)
   {
     ecrit_rapport("search_type_128 lba=%lu\n",(long unsigned)(partition->part_offset/disk_car->sector_size));
   }
   /* Test ReiserFS */
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+128*512)!=0) /* 64k offset */
+  if(disk_car->read(disk_car,9*DEFAULT_SECTOR_SIZE, buffer, partition->part_offset+128*512)!=0) /* 64k offset */
     return -1;
-  if(recover_rfs(disk_car,(const struct reiserfs_super_block*)&buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_rfs(disk_car,(const struct reiserfs_super_block*)buffer,partition,debug,dump_ind)==0) return 1;
   /* Test UFS2 */
-  if(recover_ufs(disk_car,(const struct ufs_super_block*)&buffer,partition,debug,dump_ind)==0) return 1;
+  if(recover_ufs(disk_car,(const struct ufs_super_block*)buffer,partition,debug,dump_ind)==0) return 1;
   return 0;
 }
 
@@ -283,8 +301,8 @@ t_list_part *search_superblock(t_param_disk *disk_car, const t_partition *partit
       }
     }
   }
-  FREE(new_partition);
-  FREE(buffer);
+  free(new_partition);
+  free(buffer);
   return list_part;
 }
 
