@@ -2,7 +2,7 @@
 
     File: bsd.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,14 +15,17 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
  
 #include "types.h"
 #include "common.h"
@@ -31,14 +34,22 @@
 #include "fnctdsk.h"
 int check_BSD(t_param_disk *disk_car,t_partition *partition,const int debug, const unsigned int max_partitions)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset))
-  { return 1; }
-  if(test_BSD(disk_car,(const struct disklabel*)&buffer[0x200],partition,debug,0,max_partitions))
+  unsigned char *buffer;
+  buffer=(unsigned char*)MALLOC(BSD_DISKLABEL_SIZE);
+  if(disk_car->read(disk_car,BSD_DISKLABEL_SIZE, buffer, partition->part_offset+0x200))
+  {
+    free(buffer);
     return 1;
-  set_part_name(partition,((const struct disklabel*)(&buffer[0x200]))->d_packname,16);
+  }
+  if(test_BSD(disk_car,(const struct disklabel*)buffer,partition,debug,0,max_partitions))
+  {
+    free(buffer);
+    return 1;
+  }
+  set_part_name(partition,((const struct disklabel*)buffer)->d_packname,16);
   if(check_volume_name(partition->name,16))
     partition->name[0]='\0';
+  free(buffer);
   return 0;
 }
 
