@@ -2,7 +2,7 @@
 
     File: bsd.h
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2004,2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,15 +15,16 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 /* Come mainly from sys/disklabel.h and disktab.h */
 
 #ifndef _BSD_H
 #define _BSD_H
-
+/* BSD_DISKLABEL_SIZE is 276 */
+#define BSD_DISKLABEL_SIZE 512
 #define STANDALONE
 
 #define BBSIZE		8192	/* size of boot area, with label */
