@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -46,30 +46,33 @@ static unsigned int up2power_aux(const unsigned int number);
 void *MALLOC(size_t size)
 {
   void *res;
-  if(size==0)
+  if(size<=0)
   {
     ecrit_rapport("Try to allocate 0 byte of memory\n");
     exit(EXIT_FAILURE);
   }
+#ifdef HAVE_POSIX_MEMALIGN
+  if(size>=512)
+  {
+    /* Warning, memory leak checker must be posix_memalign aware, otherwise reports may look strange */
+    if(posix_memalign(&res,4096,size)!=0)
+    {
+      ecrit_rapport("posix_memalign failed\n");
+      exit(EXIT_FAILURE);
+    }
+    memset(res,0,size);
+    return res;
+  }
+#endif
   if((res=malloc(size))==NULL)
   {
-    ecrit_rapport(msg_MALLOC_ERROR);
+    ecrit_rapport("\nCan't allocate %u bytes of memory.\n",size);
     exit(EXIT_FAILURE);
   }
   memset(res,0,size);
   return res;
 }
 
-void FREE(void *ptr)
-{
-  if(ptr==NULL)
-  {
-    wdoprintf(stdscr,msg_FREE_ERROR);
-    exit(EXIT_FAILURE);
-  }
-  free(ptr);
-}
-
 #ifndef HAVE_SNPRINTF
 int snprintf(char *str, size_t size, const char *format, ...)
 {
@@ -176,38 +179,35 @@ void set_part_name(t_partition *partition,const char *src,const int max_size)
   partition->name[i--]='\0';
 }
 
-FILE *create_file(const char *filename)
+void create_dir(const char *dir_name, const unsigned int is_dir_name)
 {
-  FILE *f_out;
-  f_out=fopen(filename,"wb");
-  if(!f_out)
+  /* create all sub-directories */
+  char *pos;
+  char *path=strdup(dir_name);
+  if(path==NULL)
+    return;
+  pos=path+1;
+  do
   {
-    /* create all sub-directories */
-    char *pos;
-    char *path=strdup(filename);
-    if(path==NULL)
-      return f_out;
-    pos=path+1;
-    do
+    strcpy(path,dir_name);
+    pos=strchr(pos+1,'/');
+    if(pos!=NULL)
+      *pos='\0';
+    if(pos!=NULL || is_dir_name!=0)
     {
-      strcpy(path,filename);
-      pos=strchr(pos+1,'/');
-      if(pos!=NULL)
-      {
-	*pos='\0';
+#ifdef __CYGWIN__
+      if(memcmp(&path[1],":/cygdrive",10)!=0)
+#endif
 #ifdef HAVE_MKDIR
 #ifdef __MINGW32__
-	mkdir(path);
+      mkdir(path);
 #else
-	mkdir(path, 0775);
+      mkdir(path, 0775);
 #endif
 #else
 #error You need a mkdir function!
 #endif
-      }
-    } while(pos!=NULL);
-    FREE(path);
-    f_out=fopen(filename,"wb");
-  }
-  return f_out;
+    }
+  } while(pos!=NULL);
+  free(path);
 }
