@@ -15,10 +15,11 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+
 #define DEFAULT_SECTOR_SIZE     0x200
 
 #define MAX_CYLINDERS 65535
@@ -228,7 +229,7 @@ typedef struct my_data_struct t_my_data;
 struct my_data_struct
 {
   t_param_disk *disk_car;
-  t_partition *partition;
+  const t_partition *partition;
   uint64_t offset;
 };
 
@@ -236,7 +237,6 @@ void dup_t_partition(t_partition *dest, const t_partition *src);
 int read_line(void);
 char read_char(const char *);
 void *MALLOC(size_t size);
-void FREE(void *ptr);
 char read_key(void);
 char test_key(void);
 unsigned int up2power(const unsigned int number);
@@ -304,10 +304,29 @@ int rebuild_FAT_BS(t_param_disk *disk_car,t_partition *partition, const int debu
 int snprintf(char *str, size_t size, const char *format, ...);
 #endif
 #ifndef HAVE_VSNPRINTF
+#include <stdarg.h>
 int vsnprintf(char *str, size_t size, const char *format, va_list ap);
 #endif
 #ifndef HAVE_STRNCASECMP
 int strncasecmp(const char * s1, const char * s2, size_t len);
 #endif
-FILE *create_file(const char *filename);
+void create_dir(const char *dir_name, const unsigned int is_dir_name);
+
+#ifdef DJGPP
+    #define TESTDISK_OS "Dos version"
+#elif defined(TARGET_BSD)
+    #define TESTDISK_OS "BSD version"
+#elif defined(TARGET_LINUX)
+    #define TESTDISK_OS "Linux version"
+#elif defined(TARGET_SOLARIS)
+    #define TESTDISK_OS "Solaris version"
+#elif defined(__CYGWIN__) || defined(__MINGW32__)
+    #define TESTDISK_OS "Windows version"
+#elif defined(__APPLE__)
+    #define TESTDISK_OS "Apple version"
+#elif defined(__OS2__)
+    #define TESTDISK_OS "OS2 version"
+#else
+    #define TESTDISK_OS "Undefined OS"
+#endif
 
