@@ -2,7 +2,7 @@
 
     File: cramfs.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -24,6 +24,9 @@
 #include <config.h>
 #endif
  
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -37,14 +40,19 @@ static int test_cramfs(const t_param_disk *disk_car, const struct cramfs_super *
 
 int check_cramfs(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
-  unsigned char buffer[3*DEFAULT_SECTOR_SIZE];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
-  { return 1; }
-  if(test_cramfs(disk_car,(struct cramfs_super*)&buffer[0x200],partition,debug,0)==0)
+  unsigned char *buffer=(unsigned char*)MALLOC(CRAMFS_SUPERBLOCK_SIZE);
+  if(disk_car->read(disk_car,CRAMFS_SUPERBLOCK_SIZE, buffer, partition->part_offset+0x200)!=0)
   {
-    set_cramfs_info(disk_car,(struct cramfs_super*)&buffer[0x200],partition,debug,0);
+    free(buffer);
+    return 1;
+  }
+  if(test_cramfs(disk_car,(struct cramfs_super*)buffer,partition,debug,0)==0)
+  {
+    set_cramfs_info(disk_car,(struct cramfs_super*)buffer,partition,debug,0);
+    free(buffer);
     return 0;
   }
+  free(buffer);
   return 1;
 }
 
