@@ -2,7 +2,7 @@
 
     File: ext2_dir.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,15 +15,14 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
  
-#ifdef HAVE_LIBEXT2FS
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -33,14 +32,19 @@
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
+
+#ifdef HAVE_LIBEXT2FS
 #include "ext2fs/ext2_fs.h"
 #include "ext2fs/ext2fs.h"
+#endif
+
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
 #include "dir.h"
 #include "ext2_dir.h"
 
+#ifdef HAVE_LIBEXT2FS
 #define DIRENT_DELETED_FILE	4
 /*
  * list directory
@@ -66,8 +70,8 @@ static errcode_t my_read_blk(io_channel channel, unsigned long block, int count,
 static errcode_t my_write_blk(io_channel channel, unsigned long block, int count, const void *buf);
 static errcode_t my_flush(io_channel channel);
 
-
 static io_channel alloc_io_channel(t_param_disk *disk_car,t_my_data *my_data);
+static void dir_partition_ext2_close(t_dir_data *dir_data);
 
 static struct struct_io_manager my_struct_manager = {
         magic: EXT2_ET_MAGIC_IO_MANAGER,
@@ -112,7 +116,7 @@ static io_channel alloc_io_channel(t_param_disk *disk_car,t_my_data *my_data)
   ioch->manager = my_io_manager;
   ioch->name = (char *)MALLOC(strlen(my_data->partition->name)+1);
   if (ioch->name==NULL) {
-	  FREE(ioch);
+	  free(ioch);
 	  return NULL;
   }
   memcpy(ioch->name, my_data->partition->name,strlen(my_data->partition->name)+1);
@@ -137,9 +141,9 @@ static errcode_t my_open(const char *dev, int flags, io_channel *channel)
 
 static errcode_t my_close(io_channel channel)
 {
-  FREE(channel->private_data);
-  FREE(channel->name);
-  FREE(channel);
+  free(channel->private_data);
+  free(channel->name);
+  free(channel);
 #ifdef DEBUG
   ecrit_rapport("my_close done\n");
 #endif
@@ -212,13 +216,13 @@ static int list_dir_proc2(ext2_ino_t dir,
   ino = dirent->inode;
   if (ino) {
     errcode_t retval;
-    ecrit_rapport("ext2fs_read_inode(ino=%u)\n",ino);
+//    ecrit_rapport("ext2fs_read_inode(ino=%u)\n",ino);
     if ((retval=ext2fs_read_inode(ls->current_fs,ino, &inode))!=0)
     {
       ecrit_rapport("ext2fs_read_inode(ino=%u) failed with error %ld.\n",(unsigned)ino, (long)retval);
       memset(&inode, 0, sizeof(struct ext2_inode));
       /*
-      FREE(new_file);
+      free(new_file);
       return 0;
       */
     }
@@ -262,77 +266,54 @@ static t_file_data *ext2_dir(t_param_disk *disk_car, const t_partition *partitio
   return ls->dir_list;
 }
 
-int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
+static void dir_partition_ext2_close(t_dir_data *dir_data)
 {
-  errcode_t       retval;
-  static struct list_dir_struct ls;
+  struct list_dir_struct *ls=(struct list_dir_struct *)dir_data->private_dir_data;
+  ext2fs_close (ls->current_fs);
+  /* ext2fs_close call the close function that freed my_data */
+  free(ls);
+}
+#endif
+
+int dir_partition_ext2_init(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const int debug)
+{
+#ifdef HAVE_LIBEXT2FS
+  struct list_dir_struct *ls=(struct list_dir_struct *)MALLOC(sizeof(*ls));
   io_channel ioch;
   t_my_data *my_data;
-  ls.dir_list=NULL;
-  ls.current_file=NULL;
-  aff_buffer(BUFFER_RESET,"Q");
-  aff_copy(window);
-  wmove(window,4,0);
-  aff_part(window,AFF_PART_ORDER,disk_car,partition);
+  ls->dir_list=NULL;
+  ls->current_file=NULL;
+  /*  ls->flags = DIRENT_FLAG_INCLUDE_EMPTY; */
+  ls->flags = 0;
   my_data=MALLOC(sizeof(*my_data));
   my_data->partition=partition;
   my_data->disk_car=disk_car;
-
   ioch=alloc_io_channel(disk_car,my_data);
   shared_ioch=&ioch;
-  retval = ext2fs_open ("/dev/testdisk", 0, partition->boot_sector, partition->blocksize, my_io_manager, &ls.current_fs);
-  if(retval!=0)
+  if(ext2fs_open ("/dev/testdisk", 0, partition->boot_sector, partition->blocksize, my_io_manager, &ls->current_fs)!=0)
   {
-    aff_part_rapport(disk_car,partition);
-    aff_buffer(BUFFER_ADD,"Can't open ext2 filesystem (retval=%ld)\n",(long)retval);
-    screen_buffer_display(window,"",NULL);
+    free(ls);
     return -1;
   }
-/*  ls.flags = DIRENT_FLAG_INCLUDE_EMPTY; */
-  ls.flags = 0;
-  {
-    t_dir_data dir_data;
-    dir_data.window=window;
-    dir_data.debug=debug;
-    dir_data.private_dir_data=&ls;
-    dir_data.get_dir=ext2_dir;
-    dir_data.copy_file=NULL;
-    strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
-    dir_partition(disk_car,partition,&dir_data,EXT2_ROOT_INO,current_cmd);
-  }
-  ext2fs_close (ls.current_fs);
+  strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
+  dir_data->current_inode=EXT2_ROOT_INO;
+  dir_data->debug=debug;
+  dir_data->get_dir=ext2_dir;
+  dir_data->copy_file=NULL;
+  dir_data->close=&dir_partition_ext2_close;
+  dir_data->local_dir=NULL;
+  dir_data->private_dir_data=ls;
   return 0;
+#else
+  return -2;
+#endif
 }
 
 const char*td_ext2fs_version(void)
 {
   const char *ext2fs_version="none";
+#ifdef HAVE_LIBEXT2FS
   ext2fs_get_library_version(&ext2fs_version,NULL);
+#endif
   return ext2fs_version;
 }
-#else
-#include "types.h"
-#include "common.h"
-#include "intrf.h"
-#include "ext2_dir.h"
-
-int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
-{
-  aff_buffer(BUFFER_RESET,"Q");
-  aff_copy(window);
-  wmove(window,4,0);
-  aff_part(window,AFF_PART_ORDER,disk_car,partition);
-  aff_part_rapport(disk_car,partition);
-  aff_buffer(BUFFER_ADD,"Recompile with e2fsprogs library");
-  if(*current_cmd!=NULL)
-    screen_buffer_to_log();
-  else
-    screen_buffer_display(window,"",NULL);
-  return 0;
-}
-
-const char*td_ext2fs_version(void)
-{
-  return "none";
-}
-#endif
