@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -276,11 +276,13 @@ int dump_2fat_rapport(const struct fat_boot_sector*fh1, const struct fat_boot_se
 
 int check_FAT(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
-  unsigned char buffer[3*disk_car->sector_size];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  unsigned char *buffer;
+  buffer=MALLOC(3*disk_car->sector_size);
+  if(disk_car->read(disk_car,3*disk_car->sector_size, buffer, partition->part_offset)!=0)
   {
     aff_buffer(BUFFER_ADD,msg_CHKFAT_RERR);
     ecrit_rapport(msg_CHKFAT_RERR);
+    free(buffer);
     return 1;
   }
   if(test_FAT(disk_car,(const struct fat_boot_sector *)buffer,partition,debug,0)!=0)
@@ -291,10 +293,12 @@ int check_FAT(t_param_disk *disk_car,t_partition *partition,const int debug)
       aff_part_rapport(disk_car,partition);
       dump_fat_rapport((const struct fat_boot_sector*)buffer, partition->upart_type,disk_car->sector_size);
     }
+    free(buffer);
     return 1;
   }
   set_FAT_info(disk_car,(const struct fat_boot_sector *)buffer,partition,debug);
 /*  aff_buffer(BUFFER_ADD,"Ok\n"); */
+  free(buffer);
   return 0;
 }
 
@@ -361,7 +365,7 @@ unsigned int get_next_cluster(t_param_disk *disk_car,const t_partition *partitio
 	unsigned long int offset_s,offset_o;
 	offset_s=cluster/(disk_car->sector_size/2);
 	offset_o=cluster%(disk_car->sector_size/2);
-	if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
+	if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
 	{
 	  ecrit_rapport("\nget_next_cluster error"); return 0;
 	}
@@ -374,7 +378,7 @@ unsigned int get_next_cluster(t_param_disk *disk_car,const t_partition *partitio
 	unsigned long int offset_s,offset_o;
 	offset_s=cluster/(disk_car->sector_size/4);
 	offset_o=cluster%(disk_car->sector_size/4);
-	if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
+	if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
 	{
 	  ecrit_rapport("\nget_next_cluster error"); return 0;
 	}
@@ -468,7 +472,7 @@ unsigned int fat32_get_prev_cluster(t_param_disk *disk_car,const t_partition *pa
     offset_o=prev_cluster%(disk_car->sector_size/4);
     if((offset_o==0)||(prev_cluster==2))
     {
-      if(disk_car->read(disk_car,sizeof(buffer), &buffer, hd_offset)!=0)
+      if(disk_car->read(disk_car,disk_car->sector_size, &buffer, hd_offset)!=0)
       {
 	ecrit_rapport("fat32_get_prev_cluster error\n"); return 0;
       }
@@ -574,6 +578,7 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
   }
   if(no_of_cluster<4085)
   {
+    upart_type=UP_FAT12;
     if(debug>0)
     {
       ecrit_rapport("FAT12\n");
@@ -608,10 +613,10 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
       aff_buffer(BUFFER_ADD,"Should be marked as FAT12\n");
       ecrit_rapport("Should be marked as FAT12\n");
     }
-    upart_type=UP_FAT12;
   }
   else if(no_of_cluster<65525)
   {
+    upart_type=UP_FAT16;
     if(debug>0)
     {
       ecrit_rapport("FAT16\n");
@@ -641,10 +646,10 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
       aff_buffer(BUFFER_ADD,"Should be marked as FAT16\n");
       ecrit_rapport("Should be marked as FAT16\n");
     }
-    upart_type=UP_FAT16;
   }
   else
   {
+    upart_type=UP_FAT32;
     if(debug>0)
     {
       ecrit_rapport("FAT32\n");
@@ -680,7 +685,6 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
       aff_buffer(BUFFER_ADD,"Should be marked as FAT32\n");
       ecrit_rapport("Should be marked as FAT32\n");
     }
-    upart_type=UP_FAT32;
   }
   if(partition->part_size>0)
   {
@@ -716,16 +720,16 @@ int test_FAT(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_
   }
   if(fat_header->fats>1)
     comp_FAT(disk_car,partition,fat_length,le16(fat_header->reserved));
-  if(le16(fat_header->secs_track)!=disk_car->CHS.sector)
-  {
-    aff_buffer(BUFFER_ADD,"check_FAT: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
-    ecrit_rapport("sect/track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
-  }
   if(le16(fat_header->heads)!=disk_car->CHS.head+1)
   {
-    aff_buffer(BUFFER_ADD,"check_FAT: Incorrect number of heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
+    aff_buffer(BUFFER_ADD,"Warning: Incorrect number of heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
     ecrit_rapport("heads/cylinder %u (FAT) != %u (HD)\n",le16(fat_header->heads),disk_car->CHS.head+1);
   }
+  if(le16(fat_header->secs_track)!=disk_car->CHS.sector)
+  {
+    aff_buffer(BUFFER_ADD,"Warning: Incorrect number of sectors per track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
+    ecrit_rapport("sect/track %u (FAT) != %u (HD)\n",le16(fat_header->secs_track),disk_car->CHS.sector);
+  }
   return 0;
 }
 
@@ -736,8 +740,10 @@ int comp_FAT(t_param_disk *disk_car,const t_partition *partition,const unsigned
   */
   unsigned int reste=taille;
   unsigned int read_size;
-  unsigned char buffer[NBR_SECT*disk_car->sector_size],buffer2[NBR_SECT*disk_car->sector_size];
   uint64_t hd_offset, hd_offset2;
+  unsigned char *buffer, *buffer2;
+  buffer=(unsigned char *)MALLOC(NBR_SECT*disk_car->sector_size);
+  buffer2=(unsigned char *)MALLOC(NBR_SECT*disk_car->sector_size);
   hd_offset=partition->part_offset+(uint64_t)sect_res*disk_car->sector_size;
   hd_offset2=hd_offset+(uint64_t)taille*disk_car->sector_size;
   if(reste>1000) reste=1000;	/* Quick check ! */
@@ -745,10 +751,10 @@ int comp_FAT(t_param_disk *disk_car,const t_partition *partition,const unsigned
   {
     read_size=reste>NBR_SECT?NBR_SECT:reste;
     reste-=read_size;
-    if(disk_car->read(disk_car,read_size*disk_car->sector_size, &buffer, hd_offset))
+    if(disk_car->read(disk_car,read_size*disk_car->sector_size, buffer, hd_offset))
     { ecrit_rapport(msg_CHKFAT_RERR);
       return 1;}
-    if(disk_car->read(disk_car,read_size*disk_car->sector_size, &buffer2, hd_offset2))
+    if(disk_car->read(disk_car,read_size*disk_car->sector_size, buffer2, hd_offset2))
     { ecrit_rapport(msg_CHKFAT_RERR); 
       return 1;}
     if(memcmp(buffer,buffer2,disk_car->sector_size*read_size)!=0)
@@ -757,12 +763,15 @@ int comp_FAT(t_param_disk *disk_car,const t_partition *partition,const unsigned
 	(unsigned long) ((hd_offset-partition->part_offset)/disk_car->sector_size-sect_res),
 	(unsigned long) ((hd_offset-partition->part_offset)/disk_car->sector_size-sect_res+read_size),
 	taille); 
-
+      free(buffer2);
+      free(buffer);
       return 1;
     }
     hd_offset+=read_size*disk_car->sector_size;
     hd_offset2+=read_size*disk_car->sector_size;
   }
+  free(buffer2);
+  free(buffer);
   return 0;
 }
 
@@ -879,7 +888,7 @@ int fat32_set_part_name(t_param_disk *disk_car, t_partition *partition, const st
 	}
       }
     }
-    FREE(buffer);
+    free(buffer);
   }
   if(partition->name[0]=='\0')
   {
@@ -894,7 +903,7 @@ int fat32_set_part_name(t_param_disk *disk_car, t_partition *partition, const st
 int check_HPFS(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
   unsigned char buffer[disk_car->sector_size];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset)!=0)
   {
     aff_buffer(BUFFER_ADD,"check_HPFS: Read error\n");
     ecrit_rapport("check_HPFS: Read error\n");
@@ -951,7 +960,7 @@ int recover_HPFS(t_param_disk *disk_car,const struct fat_boot_sector*fat_header,
 int check_OS2MB(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
   unsigned char buffer[disk_car->sector_size];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset)!=0)
   {
     aff_buffer(BUFFER_ADD,"check_OS2MB: Read error\n");
     ecrit_rapport("check_OS2MB: Read error\n");
@@ -1052,10 +1061,12 @@ int is_fat32(const t_partition *partition)
 
 int fat32_free_info(t_param_disk *disk_car,const t_partition *partition, const unsigned int fat_offset, const unsigned int no_of_cluster, unsigned int *next_free, unsigned int*free_count)
 {
-  unsigned char buffer[disk_car->sector_size];
-  uint32_t *p32=(uint32_t*)&buffer;
-  uint64_t hd_offset=partition->part_offset+(uint64_t)fat_offset*disk_car->sector_size;
+  unsigned char *buffer;
+  uint32_t *p32;
   unsigned int prev_cluster;
+  uint64_t hd_offset=partition->part_offset+(uint64_t)fat_offset*disk_car->sector_size;
+  buffer=(unsigned char *)MALLOC(disk_car->sector_size);
+  p32=(uint32_t*)buffer;
   *next_free=0;
   *free_count=0;
   for(prev_cluster=2;prev_cluster<=no_of_cluster+1;prev_cluster++)
@@ -1066,14 +1077,14 @@ int fat32_free_info(t_param_disk *disk_car,const t_partition *partition, const u
     offset_o=prev_cluster%(disk_car->sector_size/4);
     if((offset_o==0)||(prev_cluster==2))
     {
-      if(disk_car->read(disk_car,sizeof(buffer), &buffer, hd_offset)!=0)
+      if(disk_car->read(disk_car,disk_car->sector_size, buffer, hd_offset)!=0)
       {
 	ecrit_rapport("fat32_free_info read error\n");
 	*next_free=0xFFFFFFFF;
 	*free_count=0xFFFFFFFF;
 	return 1;
       }
-      hd_offset+=sizeof(buffer);
+      hd_offset+=disk_car->sector_size;
     }
     cluster=le32(p32[offset_o]) & 0xFFFFFFF;
     if(cluster==0)
@@ -1084,6 +1095,7 @@ int fat32_free_info(t_param_disk *disk_car,const t_partition *partition, const u
     }
   }
   ecrit_rapport("next_free %u, free_count %u\n",*next_free,*free_count);
+  free(buffer);
   return 0;
 }
 
