@@ -2,7 +2,7 @@
 
     File: fat_adv.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -45,7 +45,9 @@
 #include "testdisk.h"
 #include "intrf.h"
 #include "dir.h"
+#include "dirpart.h"
 #include "fat_dir.h"
+#include "io_redir.h"
 static const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 
@@ -81,7 +83,7 @@ static int fat_find_type(t_param_disk *disk_car,const t_partition *partition,con
 static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_fat12, const int p_fat16, const int p_fat32,unsigned long int*fat_offset, const unsigned int sector_size);
 
 static upart_type_t no_of_cluster2part_type(const unsigned long int no_of_cluster);
-static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd);
+static int create_fat_boot_sector(t_param_disk *disk_car, t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd);
 static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_partition *partition,const unsigned int cluster_size, const unsigned long int no_of_cluster, const unsigned int reserved, const unsigned int fat_length, const int interface, const int debug, const unsigned int expert, const unsigned int first_free_cluster, const unsigned int fats);
 static int write_FAT_boot_code_aux(unsigned char *buffer);
 static int find_cluster_size(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind,const int interface, unsigned int *cluster_size, unsigned long int *offset);
@@ -213,6 +215,8 @@ static int ask_root_directory(t_param_disk *disk_car, const t_partition *partiti
       char		datestr[80];
       wmove(window,8+i-offset,0);
       wclrtoeol(window);	/* before addstr for BSD compatibility */
+      if(current_file==pos)
+	wattrset(window,A_STANDOUT);
       if(current_file->filestat.st_mtime!=0)
       {
 	tm_p = localtime(&current_file->filestat.st_mtime);
@@ -224,8 +228,6 @@ static int ask_root_directory(t_param_disk *disk_car, const t_partition *partiti
 	strncpy(datestr, "                 ",sizeof(datestr));
       }
       mode_string(current_file->filestat.st_mode,str);
-      if(current_file==pos)
-	wattrset(window,A_STANDOUT);
       wdoprintf(window, "%s %5u %5u   ", 
 	  str, (unsigned int)current_file->filestat.st_uid, (unsigned int)current_file->filestat.st_gid);
       wdoprintf(window, "%7llu", (long long unsigned int)current_file->filestat.st_size);
@@ -251,7 +253,8 @@ static int ask_root_directory(t_param_disk *disk_car, const t_partition *partiti
       case 'Y':
       case 'n':
       case 'N':
-      quit=1;
+	quit=1;
+	break;
     }
     if(dir_list!=NULL)
     {
@@ -349,8 +352,40 @@ static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_parti
 	     dump(stdscr,buffer,cluster_size*disk_car->sector_size);
 	   */
 	}
-	if(memcmp(&buffer[0],".          ",8+3)!=0 || memcmp(&buffer[0*0x20],&buffer[1*0x20],0x20)!=0)
-	{
+	if((memcmp(&buffer[0],".          ",8+3)==0) &&
+	  (buffer[0xB]!=ATTR_EXT && (buffer[0xB]&ATTR_DIR)!=0))
+	{ /* Directory found */
+	  unsigned long int cluster=(buffer[1*0x20+0x15]<<24)+(buffer[1*0x20+0x14]<<16)+
+	    (buffer[1*0x20+0x1B]<<8)+buffer[1*0x20+0x1A];
+	  if((memcmp(&buffer[0x20],"..         ",8+3)==0) &&
+	      (buffer[1*0x20+0xB]!=ATTR_EXT && (buffer[1*0x20+0xB]&ATTR_DIR)!=0) && (cluster==0)
+	      && (buffer[0x40]!=0)) /* First-level directory */
+	  {
+	    t_file_data *dir_list;
+	    ecrit_rapport("First-level directory found at cluster %lu\n",root_cluster);
+	    /*    dump(stdscr,buffer,cluster_size*disk_car->sector_size); */
+	    dir_list=dir_fat_aux(buffer,cluster_size*disk_car->sector_size,cluster_size);
+	    if(debug>0)
+	    {
+	      dir_aff_log(disk_car, partition, NULL, dir_list);
+	    }
+	    {
+	      t_file_data *new_file=MALLOC(sizeof(*new_file));
+	      memcpy(new_file,dir_list,sizeof(*new_file));
+	      new_file->prev=current_file;
+	      new_file->next=NULL;
+	      if(current_file!=NULL)
+		current_file->next=new_file;
+	      else
+		rootdir_list=new_file;
+	      current_file=new_file;
+	      snprintf(new_file->name,sizeof(new_file->name),"DIR%05u",++dir_nbr);
+	    }
+	    delete_list_file(dir_list);
+	  }
+	}
+	else if( memcmp(&buffer[0*0x20],&buffer[1*0x20],0x20)!=0)
+	{	/* Potential root directory */
 	  unsigned int i,found=1;
 	  int etat=0,nb_subdir=0,nb_subdir_ok=0;
 	  for(i=0;found && (i<cluster_size*disk_car->sector_size/0x20);i++)
@@ -476,37 +511,7 @@ static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_parti
 	    }
 	  }
 	}
-	else
-	{ /* Directory found */
-	  unsigned long int cluster=(buffer[1*0x20+0x15]<<24)+(buffer[1*0x20+0x14]<<16)+
-	    (buffer[1*0x20+0x1B]<<8)+buffer[1*0x20+0x1A];
-	  if((memcmp(&buffer[0x20],"..         ",8+3)==0) &&
-	      (buffer[1*0x20+0xB]!=ATTR_EXT && (buffer[1*0x20+0xB]&ATTR_DIR)!=0) && (cluster==0)
-	      && (buffer[0x40]!=0)) /* First-level directory */
-	  {
-	    t_file_data *dir_list;
-	    ecrit_rapport("First-level directory found at cluster %lu\n",root_cluster);
-	    /*    dump(stdscr,buffer,cluster_size*disk_car->sector_size); */
-	    dir_list=dir_fat_aux(buffer,cluster_size*disk_car->sector_size,cluster_size);
-	    if(debug>0)
-	    {
-	      dir_aff_log(disk_car, partition, NULL, dir_list);
-	    }
-	    {
-	      t_file_data *new_file=MALLOC(sizeof(*new_file));
-	      memcpy(new_file,dir_list,sizeof(*new_file));
-	      new_file->prev=current_file;
-	      new_file->next=NULL;
-	      if(current_file!=NULL)
-		current_file->next=new_file;
-	      else
-		rootdir_list=new_file;
-	      current_file=new_file;
-	      snprintf(new_file->name,sizeof(new_file->name),"DIR%05u",++dir_nbr);
-	    }
-	    delete_list_file(dir_list);
-	  }
-	}
+
       }
     }
     if(ind_stop!=0)
@@ -644,7 +649,7 @@ static int fat32_create_rootdir(t_param_disk *disk_car,const t_partition *partit
     delete_list_file(dir_list);
   }
 #endif
-  FREE(buffer);
+  free(buffer);
   return 0;
 }
 
@@ -713,7 +718,7 @@ static int analyse_dir_entries(t_param_disk *disk_car,const t_partition *partiti
   hd_offset=partition->part_offset+(uint64_t)offset*disk_car->sector_size;
   for(i=0;(i<200)&&(etat!=2);i++)
   {
-    if(disk_car->read(disk_car,sizeof(buffer), &buffer, hd_offset)!=0)
+    if(disk_car->read(disk_car,disk_car->sector_size, &buffer, hd_offset)!=0)
     {
       ecrit_rapport("dir_entries: read error, dir_entries>=%u (%u sectors)\n",i*(disk_car->sector_size/32),i);
     }
@@ -750,7 +755,7 @@ static int analyse_dir_entries(t_param_disk *disk_car,const t_partition *partiti
 	  }
       }
     }
-    hd_offset+=sizeof(buffer);
+    hd_offset+=disk_car->sector_size;
   }
   if(i==200)
     return 0;
@@ -772,7 +777,7 @@ static int analyse_dir_entries2(t_param_disk *disk_car,const t_partition *partit
   if(disk_car->read(disk_car, nbr_sector*disk_car->sector_size, buffer_dir, partition->part_offset+(uint64_t)(reserved+fats*fat_length)*disk_car->sector_size)!=0)
   {
     ecrit_rapport("FAT 1x can't read root directory\n");
-    FREE(buffer_dir);
+    free(buffer_dir);
     return 0;
   }
   {
@@ -815,7 +820,7 @@ static int analyse_dir_entries2(t_param_disk *disk_car,const t_partition *partit
 	  }
 	  if(cluster_prev==0 && cluster==new_inode)
 	  {
-	    FREE(buffer_dir);
+	    free(buffer_dir);
 	    delete_list_file(dir_list);
 	    return ((dir_entries+(disk_car->sector_size/32)-1)/(disk_car->sector_size/32))*(disk_car->sector_size/32);
 	  }
@@ -827,12 +832,12 @@ static int analyse_dir_entries2(t_param_disk *disk_car,const t_partition *partit
   {
     ecrit_rapport("No directory found\n");
   }
-  FREE(buffer_dir);
+  free(buffer_dir);
   delete_list_file(dir_list);
   return root_size_max;
 }
 
-static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd)
+static int create_fat_boot_sector(t_param_disk *disk_car, t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd)
 {
   unsigned char orgboot[3*disk_car->sector_size];
   unsigned char newboot[3*disk_car->sector_size];
@@ -840,12 +845,12 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
   struct fat_boot_sector *fat_header=(struct fat_boot_sector *)&newboot;
   int error=0;
   unsigned long int part_size=0;
-  if(disk_car->read(disk_car,sizeof(orgboot), &orgboot, partition->part_offset)!=0)
+  if(disk_car->read(disk_car,3*disk_car->sector_size, &orgboot, partition->part_offset)!=0)
   {
     ecrit_rapport("create_fat_boot_sector: Can't read old boot sector\n");
-    memset(&orgboot,0,sizeof(orgboot));
+    memset(&orgboot,0,3*disk_car->sector_size);
   }
-  memcpy(&newboot,&orgboot,sizeof(newboot));
+  memcpy(&newboot,&orgboot,3*disk_car->sector_size);
   if((le16(fat_header->marker)!=0xAA55)||
       !((fat_header->ignored[0]==0xeb && fat_header->ignored[2]==0x90)||fat_header->ignored[0]==0xe9))
   {
@@ -1030,6 +1035,7 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
     const char *options="DLQ";
     int do_write=0;
     int do_exit=0;
+    int no_confirm=0;
     do
     {
       int command;
@@ -1062,7 +1068,7 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
 	command='Q';
 	while(*current_cmd[0]==',')
 	  (*current_cmd)++;
-	if(strncmp(*current_cmd,"list",9)==0)
+	if(strncmp(*current_cmd,"list",4)==0)
 	{
 	  (*current_cmd)+=4;
 	  command='L';
@@ -1072,6 +1078,17 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
 	  (*current_cmd)+=4;
 	  command='D';
 	}
+	else if(strncmp(*current_cmd,"noconfirm",9)==0)
+	{
+	  command=0;	/* do nothing */
+	  no_confirm=1;
+	  (*current_cmd)+=9;
+	}
+      	else if(strncmp(*current_cmd,"write",5)==0)
+	{
+	  command='W';
+	  (*current_cmd)+=5;
+	}
 	screen_buffer_to_log();
       }
       else
@@ -1106,7 +1123,16 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
 	  break;
 	case 'l':
 	case 'L':
-	  dir_partition_fat_aux(stdscr,disk_car, partition, debug, fat_header,upart_type,current_cmd);
+	  {
+	    /* TODO: use redirector and dir_partition_fat */
+	    //dir_partition_fat_aux(stdscr,disk_car, partition, debug, fat_header,upart_type,current_cmd);
+	    const upart_type_t old_upart_type=upart_type;
+	    partition->upart_type=upart_type;
+	    io_redir_add_redir(disk_car,partition->part_offset,3*disk_car->sector_size,0,newboot);
+	    dir_partition(disk_car, partition, debug, current_cmd);
+	    io_redir_del_redir(disk_car,partition->part_offset);
+	    partition->upart_type=old_upart_type;
+	  }
 	  break;
 	case 'q':
 	case 'Q':
@@ -1114,15 +1140,15 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
 	  break;
       }
     } while(do_write==0 && do_exit==0);
-    if(do_write!=0 && ask_confirmation("Write FAT boot sector, confirm ? (Y/N)")!=0)
+    if(do_write!=0 && (no_confirm!=0 || ask_confirmation("Write FAT boot sector, confirm ? (Y/N)")!=0))
     {
       int err=0;
       ecrit_rapport("Write new boot!\n");
       /* Write boot sector and backup boot sector */
       if(upart_type==UP_FAT32)
       {
-	if(disk_car->write(disk_car,sizeof(newboot), &newboot, partition->part_offset)!=0 ||
-	    disk_car->write(disk_car,sizeof(newboot), &newboot, partition->part_offset+(uint64_t)le16(fat_header->backup_boot)*disk_car->sector_size)!=0)
+	if(disk_car->write(disk_car,3*disk_car->sector_size, &newboot, partition->part_offset)!=0 ||
+	    disk_car->write(disk_car,3*disk_car->sector_size, &newboot, partition->part_offset+(uint64_t)le16(fat_header->backup_boot)*disk_car->sector_size)!=0)
 	  err=1;
       }
       else
@@ -1452,7 +1478,7 @@ static int fat_find_type(t_param_disk *disk_car,const t_partition *partition,con
       waddstr(stdscr,"  Stop  ");
       wattroff(stdscr,A_STANDOUT);
   }
-  for(offset=disk_car->sector_size;(offset<max_offset)&&(*nbr_offset<0x200)&&!ind_stop;offset+=sizeof(buffer))
+  for(offset=disk_car->sector_size;(offset<max_offset)&&(*nbr_offset<0x200)&&!ind_stop;offset+=disk_car->sector_size)
   {
     unsigned long int percent=offset*100/max_offset;
     if(interface && (percent!=old_percent))
@@ -1464,7 +1490,7 @@ static int fat_find_type(t_param_disk *disk_car,const t_partition *partition,con
       wrefresh(stdscr);
       ind_stop|=check_enter_or_s(stdscr);
     }
-    if(disk_car->read(disk_car,sizeof(buffer), buffer, partition->part_offset+offset)==0)
+    if(disk_car->read(disk_car,disk_car->sector_size, buffer, partition->part_offset+offset)==0)
     {
       unsigned long int fat_offset=0;
       unsigned int fat_type;
@@ -1551,7 +1577,7 @@ static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved,
     if((dump_ind>0) && (interface>0))
     {
       unsigned char buffer[disk_car->sector_size];
-      if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)info_offset[i].offset*disk_car->sector_size)==0)
+      if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset+(uint64_t)info_offset[i].offset*disk_car->sector_size)==0)
       {
 	dump(stdscr,buffer,disk_car->sector_size);
       }
@@ -1777,7 +1803,7 @@ static int find_cluster_size(t_param_disk *disk_car, t_partition *partition, con
       wrefresh(stdscr);
       ind_stop|=check_enter_or_s(stdscr);
     }
-    if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+offset)==0)
+    if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset+offset)==0)
     {
       if((memcmp(&buffer[0],".          ",8+3)==0)&&(memcmp(&buffer[0x20],"..         ",8+3)==0))
       {
@@ -1788,7 +1814,7 @@ static int find_cluster_size(t_param_disk *disk_car, t_partition *partition, con
 	sector_cluster[nbr_subdir].sector=offset/disk_car->sector_size;
 	nbr_subdir++;
 	if(dump_ind>0)
-	  dump(stdscr,buffer,sizeof(buffer));
+	  dump(stdscr,buffer,disk_car->sector_size);
       }
     }
   }
@@ -2238,7 +2264,7 @@ int FAT_init_rootdir(t_param_disk *disk_car, t_partition *partition, const int d
     start_data=start_rootdir+(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size;
     for(sector=start_rootdir;error==0 && sector<start_data;sector++)
     {
-      if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)sector*disk_car->sector_size)!=0)
+      if(disk_car->read(disk_car,disk_car->sector_size, &buffer, partition->part_offset+(uint64_t)sector*disk_car->sector_size)!=0)
       {
 	ecrit_rapport("FAT_init_rootdir: read error at sector %lu\n", sector);
       }
