@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -48,9 +48,9 @@
 #include "fat_dir.h"
 
 #define MSDOS_MKMODE(a,m) ((m & (a & ATTR_RO ? LINUX_S_IRUGO|LINUX_S_IXUGO : LINUX_S_IRWXUGO)) | (a & ATTR_DIR ? LINUX_S_IFDIR : LINUX_S_IFREG))
-struct fat_private_struct
+struct list_dir_struct
 {
-  const struct fat_boot_sector*boot_sector;
+  struct fat_boot_sector*boot_sector;
 };
 
 
@@ -60,6 +60,7 @@ static t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partiti
 static t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster);
 static t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster);
 static inline void fat16_towchar(wchar_t *dst, const uint8_t *src, size_t len);
+static void dir_partition_fat_close(t_dir_data *dir_data);
 
 static int32_t secwest;
 
@@ -228,8 +229,8 @@ static int date_dos2unix(const unsigned short f_time, const unsigned short f_dat
 
 static t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster)
 {
-  const struct fat_private_struct *fat_private=(const struct fat_private_struct*)dir_data->private_dir_data;
-  const struct fat_boot_sector*fat_header=fat_private->boot_sector;
+  const struct list_dir_struct *ls=(const struct list_dir_struct*)dir_data->private_dir_data;
+  const struct fat_boot_sector*fat_header=ls->boot_sector;
   if(fat_header->cluster_size<1)
   {
     ecrit_rapport("FAT12: Can't list files, bad cluster size\n");
@@ -266,15 +267,15 @@ static t_file_data *fat12_dir(t_param_disk *disk_car, const t_partition *partiti
       }
     }
     dir_list=dir_fat_aux(buffer_dir,fat_sector_size(fat_header)*cluster_size*nbr_cluster,cluster_size);
-    FREE(buffer_dir);
+    free(buffer_dir);
     return dir_list;
   }
 }
 
 static t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster)
 {
-  const struct fat_private_struct *fat_private=(const struct fat_private_struct*)dir_data->private_dir_data;
-  const struct fat_boot_sector*fat_header=fat_private->boot_sector;
+  const struct list_dir_struct *ls=(const struct list_dir_struct*)dir_data->private_dir_data;
+  const struct fat_boot_sector*fat_header=ls->boot_sector;
   if(fat_header->cluster_size<1)
   {
     ecrit_rapport("FAT16: Can't list files, bad cluster size.\n");
@@ -312,15 +313,15 @@ static t_file_data *fat16_dir(t_param_disk *disk_car, const t_partition *partiti
       }
     }
     dir_list=dir_fat_aux(buffer_dir,disk_car->sector_size*cluster_size*nbr_cluster,cluster_size);
-    FREE(buffer_dir);
+    free(buffer_dir);
     return dir_list;
   }
 }
 
 static t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_cluster)
 {
-  const struct fat_private_struct *fat_private=(const struct fat_private_struct*)dir_data->private_dir_data;
-  const struct fat_boot_sector*fat_header=fat_private->boot_sector;
+  const struct list_dir_struct *ls=(const struct list_dir_struct*)dir_data->private_dir_data;
+  const struct fat_boot_sector*fat_header=ls->boot_sector;
   if(fat_header->cluster_size==0)
   {
     ecrit_rapport("FAT32: Can't list files, bad cluster size.\n");
@@ -363,7 +364,7 @@ static t_file_data *fat32_dir(t_param_disk *disk_car, const t_partition *partiti
     }
 //    ecrit_rapport("nbr_cluster=%u\n",nbr_cluster);
     dir_list=dir_fat_aux(buffer_dir,fat_sector_size(fat_header)*cluster_size*nbr_cluster,cluster_size);
-    FREE(buffer_dir);
+    free(buffer_dir);
     return dir_list;
   }
 }
@@ -385,11 +386,11 @@ static t_file_data *fat1x_rootdir(t_param_disk *disk_car, const t_partition *par
     if(disk_car->read(disk_car, root_size, buffer_dir, start))
     {
       ecrit_rapport("FAT 1x: Can't read root directory\n");
-      FREE(buffer_dir);
+      free(buffer_dir);
       return NULL;
     }
     res=dir_fat_aux(buffer_dir,root_size,fat_header->cluster_size);
-    FREE(buffer_dir);
+    free(buffer_dir);
     return res;
   }
 }
@@ -418,49 +419,50 @@ static void set_secwest(void)
 #endif
 }
 
-int dir_partition_fat_aux(WINDOW *window,t_param_disk *disk_car, const t_partition *partition, const int debug, struct fat_boot_sector*fat_header, const upart_type_t upart_type, char**current_cmd)
+int dir_partition_fat_init(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const int debug)
 {
-  t_dir_data dir_data;
-  struct fat_private_struct fat_private;
-  set_secwest();
-  fat_private.boot_sector=fat_header;
-  dir_data.window=window;
-  dir_data.debug=debug;
-  dir_data.private_dir_data=&fat_private;
-  switch(upart_type)
+  static unsigned char *buffer;
+  static struct list_dir_struct *ls;
+  buffer=(unsigned char*)MALLOC(0x200);
+  if(disk_car->read(disk_car,0x200, buffer, partition->part_offset+(uint64_t)partition->boot_sector*disk_car->sector_size))
   {
-    case UP_FAT12:
-      dir_data.get_dir=fat12_dir;
-      break;
-    case UP_FAT16:
-      dir_data.get_dir=fat16_dir;
-      break;
-    case UP_FAT32:
-      dir_data.get_dir=fat32_dir;
-      break;
-    default:
-      ecrit_rapport("Not a valid type (upart_type=%u)\n",partition->upart_type);
-      return -1;
-  }
-  dir_data.copy_file=NULL;
-  strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
-  return dir_partition(disk_car,partition,&dir_data,0,current_cmd);
-}
-
-int dir_partition_fat(WINDOW *window,t_param_disk *disk_car, t_partition *partition, const int debug, char **current_cmd)
-{
-  unsigned char buffer[0x200];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)partition->boot_sector*disk_car->sector_size))
-  {
-    aff_buffer(BUFFER_RESET,"Q");
-    aff_copy(window);
-    wmove(window,4,0);
-    aff_part(window,AFF_PART_ORDER,disk_car,partition);
-    aff_part_rapport(disk_car,partition);
-    aff_buffer(BUFFER_ADD,"dir_partition_fat: Can't read FAT boot sector\n");
-    screen_buffer_display(window,"",NULL);
+    free(buffer);
     return -1;
   }
-  dir_partition_fat_aux(window,disk_car, partition, debug, (struct fat_boot_sector*)buffer,partition->upart_type, current_cmd);
+  set_secwest();
+  ls=(struct list_dir_struct *)MALLOC(sizeof(*ls));
+  ls->boot_sector=(struct fat_boot_sector*)buffer;
+  strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
+  dir_data->current_inode=0;
+  dir_data->debug=debug;
+  dir_data->copy_file=NULL;
+  dir_data->close=dir_partition_fat_close;
+  dir_data->local_dir=NULL;
+  dir_data->private_dir_data=ls;
+  switch(partition->upart_type)
+  {
+    case UP_FAT12:
+      dir_data->get_dir=fat12_dir;
+      break;
+    case UP_FAT16:
+      dir_data->get_dir=fat16_dir;
+      break;
+    case UP_FAT32:
+      dir_data->get_dir=fat32_dir;
+      break;
+    default:
+      ecrit_rapport("Not a valid FAT type (upart_type=%u)\n",partition->upart_type);
+      free(ls->boot_sector);
+      free(ls);
+      dir_data->private_dir_data=NULL;
+      return -1;
+  }
   return 0;
 }
+
+static void dir_partition_fat_close(t_dir_data *dir_data)
+{
+  struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
+  free(ls->boot_sector);
+  free(ls);
+}
