@@ -2,7 +2,7 @@
 
     File: file_jpg.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -26,6 +26,7 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#include <stdio.h>
 #include "types.h"
 #ifdef HAVE_SETJMP_H
 #include <setjmp.h>
@@ -38,7 +39,6 @@
 
 static const char* header_check_jpg(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
 static void file_check_jpg(t_file_recovery *file_recovery);
-static int64_t test_jpeg(FILE *infile);
 
 const t_file_hint file_hint_jpg= {
   .extension="jpg",
@@ -101,7 +101,7 @@ static void my_error_exit (j_common_ptr cinfo)
 }
 #endif
 
-static int64_t test_jpeg(FILE *infile)
+int64_t test_jpeg(FILE *infile)
 {
   uint64_t jpeg_size=0;
 #ifdef HAVE_LIBJPEG
@@ -119,7 +119,7 @@ static int64_t test_jpeg(FILE *infile)
       /* If we get here, the JPEG code has signaled an error.
        * We need to clean up the JPEG object and return.
        */
-      /*    ecrit_rapport("JPG error at %llu\n",(long long unsigned)ftell(infile)); */
+      ecrit_rapport("JPG error at offset %llu\n",(long long unsigned)ftell(infile));
       jpeg_destroy_decompress(&cinfo);
       /* Not accurate */
       return -ftell(infile);
@@ -156,29 +156,27 @@ static int64_t test_jpeg(FILE *infile)
 #endif
   if(jpeg_size>0)
   {
-    unsigned char buffer[4096+1];
+    const unsigned int read_size=4096;
+    unsigned char buffer[read_size+1];
     int64_t jpeg_size_org=jpeg_size;
     int i;
-    buffer[4096]=0;
+    buffer[read_size]=0;
     do
     {
       int taille;
-      if(jpeg_size<4096)
-      {
-	jpeg_size=0;
-	buffer[4096]=0;
-      }
+      if(jpeg_size%read_size!=0)
+	jpeg_size=jpeg_size-(jpeg_size%read_size);
       else
-	jpeg_size-=4096;
+	jpeg_size-=read_size;
       if(fseek(infile,jpeg_size,SEEK_SET)<0)
 	return jpeg_size_org;
-      taille=fread(buffer,1,4096,infile);
+      taille=fread(buffer,1,read_size,infile);
       for(i=taille-1;i>=0;i--)
       {
 	if(buffer[i]==0xff && buffer[i+1]==0xd9)
 	  return jpeg_size+i+2;
       }
-      buffer[4096]=buffer[0];
+      buffer[read_size]=buffer[0];
     } while(jpeg_size>0);
     return jpeg_size_org;
   }
