@@ -17,10 +17,14 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+/*
+   MNG (Multiple-image Network Graphics) Format
+   http://www.libpng.org/pub/mng/spec/
+*/
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -28,10 +32,13 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
+#include <stdio.h>
 #include "types.h"
 #include "photorec.h"
 
 static const char* header_check_png(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery);
+static void file_check_png(t_file_recovery *file_recovery);
+
 const t_file_hint file_hint_png= {
   .extension="png",
   .description="Portable/JPEG/Multiple-Image Network Graphics",
@@ -41,7 +48,7 @@ const t_file_hint file_hint_png= {
   .recover=1,
   .header_check=&header_check_png,
   .data_check=NULL,
-  .file_check=NULL
+  .file_check=file_check_png
 };
 
 static const char* header_check_png(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
@@ -57,3 +64,16 @@ static const char* header_check_png(const unsigned char *buffer, const unsigned
     return "mng";
   return NULL;
 }
+
+static void file_check_png(t_file_recovery *file_recovery)
+{
+  const unsigned char mng_footer[4]= {'M','E','N','D'};
+  const unsigned char png_footer[4]= {'I','E','N','D'};
+  if(strstr(file_recovery->filename,".mng")!=NULL)
+    file_search_footer(file_recovery, mng_footer,sizeof(mng_footer));
+  else
+    file_search_footer(file_recovery, png_footer,sizeof(png_footer));
+  if(file_recovery->file_size>0)
+    file_recovery->file_size+=4;
+}
+
