@@ -15,18 +15,22 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
 #include <ctype.h>      /* tolower */
+#include <stdio.h>
 #include "types.h"
 #include "common.h"
 #include "photorec.h"
@@ -60,28 +64,48 @@ static int filtre(unsigned char car)
   switch(car)
   {
     case 0x7c:  /* similar to | */
+    case 0x80:
+    case 0x92:
+    case 0x99:
     case 0x9c:	/* '' */
+    case 0xa0:  /* nonbreaking space */
+    case 0xa1:  /* '' */
+    case 0xa2:
     case 0xa3:	/* '' */
     case 0xa7:	/* '' */
+    case 0xa8:
     case 0xa9:	/* '' */
     case 0xab:	/* '' */
     case 0xb0:	/* '' */
+    case 0xb4:  /* '' */
+    case 0xb7:
     case 0xbb:  /* '' */
+    case 0xc0:  /* '' */
+    case 0xc7:  /* '' */
     case 0xc9:  /* '' */
+    case 0xd6:  /* '' */
+    case 0xd7:
     case 0xd9:  /* '' */
+    case 0xdf:
     case 0xe0: 	/* '' */
+    case 0xe1: 	/* '' */
     case 0xe2: 	/* '' */
+    case 0xe3:  /* '' */
     case 0xe4: 	/* '' */
+    case 0xe6:  /* '' */
     case 0xe7: 	/* '' */
     case 0xe8: 	/* '' */
     case 0xe9: 	/* '' */
     case 0xea: 	/* '' */
     case 0xeb: 	/* '' */
+    case 0xed:  /* '' */
     case 0xee: 	/* '' */
     case 0xef: 	/* '' */
     case 0xf4: 	/* '' */
     case 0xf6: 	/* '' */
+    case 0xf8:  /* '' */
     case 0xf9: 	/* '' */
+    case 0xfa:  /* '' */
     case 0xfb: 	/* '' */
     case 0xfc: 	/* '' */
       return 1;
@@ -103,7 +127,7 @@ static int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, int
   unsigned char *q;	/* pointers to actual position in destination buffer */
   unsigned char bufU[8]; /* buffer for UTF8 character read from source */
   int i, l; /* counter of remaining bytes available in destination buffer, UTF8 character's length */
-  for (i = buf_len, p = buffer, q = buffer_lower; *p!='\0' && p-buffer<buf_len && i > 0;) 
+  for (i = buf_len, p = buffer, q = buffer_lower; p-buffer<buf_len && i > 0 && *p!='\0';) 
   {
     if ((*p & 0x80)!=0x80)
     {
@@ -218,7 +242,7 @@ static int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, int
 	else
 	{
 #ifdef DEBUG_FILETXT
-	  ecrit_rapport("UTF2Lat: UTF character [%s] absent in Latin1 - dropped\n", bufU);
+	  ecrit_rapport("UTF2Lat: UTF character [%s] absent in Latin1 - dropped (%c, 0x%x)\n", bufU,*q,*q);
 #endif
 	  /* There is already a null but don't care */
 	  *q = '\0';
@@ -236,7 +260,7 @@ static int UTF2Lat(unsigned char *buffer_lower, const unsigned char *buffer, int
 	else
 	{
 #ifdef DEBUG_FILETXT
-	  ecrit_rapport( "UTF2Lat: non-UTF character [%s] - dropped\n", bufU);
+	  ecrit_rapport( "UTF2Lat: non-UTF character [%s](0x%x) - dropped\n", bufU,bufU[0]);
 #endif
 	  *q = '\0';
 	  return(p_org-buffer);
@@ -260,59 +284,83 @@ static inline const unsigned char *find_in_mem(const unsigned char *haystack, co
   /*
   ecrit_rapport("\n");
   */
-  while(haystack!=NULL && haystack<haystack_end)
+  while(haystack!=NULL)
   {
     /*
     ecrit_rapport("find_in_mem(haystack=%p, haystack_end=%p,needle=%s,needle_length=%u)\n",
 	haystack,haystack_end,needle,needle_length);
     */
     haystack=memchr(haystack,needle[0],haystack_end-haystack);
-    if(haystack!=NULL)
+    if(haystack!=NULL && haystack<=(haystack_end-needle_length))
     {
       if(memcmp(haystack,needle,needle_length)==0)
 	return haystack;
       haystack++;
     }
+    else
+      haystack=NULL;
   };
   return NULL;
 }
 
 static const char* header_check_txt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only,  t_file_recovery *file_recovery)
 {
+  unsigned char *buffer_lower;
   unsigned int i;
   const unsigned char header_bat[9]  	= "@echo off";
   const unsigned char header_sh[9]  	= "#!/bin/sh";
   const unsigned char header_slk[10]  	= "ID;PSCALC3";
   const unsigned char header_rtf[5]= { '{','\\','r','t','f'};
+  const unsigned char header_imm[13]= {'M','I','M','E','-','V','e','r','s','i','o','n',':'};
+  const unsigned char header_imm2[13]={'R','e','t','u','r','n','-','P','a','t','h',':',' '};
+  const unsigned char header_mail[5]={'F','r','o','m',' '};
   const unsigned char header_asp[22]=	"<%@ language=\"vbscript";
+  const unsigned char header_ram[7]	= "rtsp://";
+  const unsigned char header_xml[14]	= "<?xml version=";
   const unsigned char header_sig_perl[4] = "perl";
   const unsigned char header_sig_python[6] = "python";
+  const unsigned char header_sig_ruby[4] = "ruby";
   const unsigned char sign_c[]		= "#include";
   const unsigned char sign_php[]	= "<?php";
   const unsigned char sign_jsp[]	= "<%@";
   const unsigned char sign_jsp2[]	= "<%=";
   const unsigned char sign_asp[]	= "<% ";
+  const unsigned char sign_grisbi[]	= "Version_grisbi";
   const unsigned char sign_html[]	= "<html";
   const unsigned char sign_h[]		= "/*";
-  unsigned char *buffer_lower;
   const unsigned char *haystack_end;
+  if(memcmp(buffer,header_mail,sizeof(header_mail))==0)
+    return "";
+  /* Incredimail */
+  if(memcmp(buffer,header_imm,sizeof(header_imm))==0 || memcmp(buffer,header_imm2,sizeof(header_imm2))==0)
+    return "imm";
   if(memcmp(buffer,header_rtf,sizeof(header_rtf))==0)
     return "rtf";
   if(memcmp(buffer,header_sh,sizeof(header_sh))==0)
     return "sh";
   if(memcmp(buffer,header_slk,sizeof(header_slk))==0)
     return "slk";
-  buffer_lower=MALLOC(buffer_size+1);
+  if(memcmp(buffer,header_ram,sizeof(header_ram))==0)
+    return "ram";
+  if(memcmp(buffer,header_xml,sizeof(header_xml))==0)
+  {
+    if(strstr(buffer,sign_grisbi)!=NULL)
+      return "gsb";
+    if(strstr(buffer,sign_html)!=NULL)
+      return "html";
+    return "xml";
+  }
+  buffer_lower=MALLOC(buffer_size+16);
   haystack_end=buffer_lower+buffer_size;
   i=UTF2Lat(buffer_lower,buffer,buffer_size);
   if(memcmp(buffer_lower,header_bat,sizeof(header_bat))==0)
   {
-    FREE(buffer_lower);
+    free(buffer_lower);
     return "bat";
   }
   if(memcmp(buffer_lower,header_asp,sizeof(header_asp))==0)
   {
-    FREE(buffer_lower);
+    free(buffer_lower);
     return "asp";
   }
   if(buffer[0]=='#' && buffer[1]=='!')
@@ -324,22 +372,25 @@ static const char* header_check_txt(const unsigned char *buffer, const unsigned
     res=memchr(haystack,'\n',haystack_size);
     if(res!=NULL)
       haystack_end=res;
-    res=find_in_mem(haystack,haystack_end,header_sig_perl,sizeof(header_sig_perl));
-    if(res!=NULL)
+    if(find_in_mem(haystack,haystack_end,header_sig_perl,sizeof(header_sig_perl)) != NULL)
     {
-      FREE(buffer_lower);
+      free(buffer_lower);
       return "pl";
     }
-    res=find_in_mem(haystack,haystack_end,header_sig_python,sizeof(header_sig_python));
-    if(res!=NULL)
+    if(find_in_mem(haystack,haystack_end,header_sig_python,sizeof(header_sig_python)) != NULL)
     {
-      FREE(buffer_lower);
+      free(buffer_lower);
       return "py";
     }
+    if(find_in_mem(haystack,haystack_end,header_sig_python,sizeof(header_sig_ruby)) != NULL)
+    {
+      free(buffer_lower);
+      return "rb";
+    }
   }
   if(safe_header_only!=0)
   {
-    FREE(buffer_lower);
+    free(buffer_lower);
     return NULL;
   }
   {
@@ -385,6 +436,7 @@ static const char* header_check_txt(const unsigned char *buffer, const unsigned
     }
     if(file_recovery!=NULL && file_recovery->file_stat!=NULL)
     {
+      const unsigned char zip_header[4]  = { 'P', 'K', 0x03, 0x04};
       /* Special case: doc, texte files
 	Unix: \n (0xA)
 	Dos: \r\n (0xD 0xA)
@@ -399,7 +451,7 @@ static const char* header_check_txt(const unsigned char *buffer, const unsigned
 	for(j=0;j<i-1;j++)
 	  if(buffer_lower[j]=='\r' && buffer_lower[j+1]!='\n')
 	  {
-	    FREE(buffer_lower);
+	    free(buffer_lower);
 	    return NULL;
 	  }
 	for(j=0;j<i && j<512;j++)
@@ -408,7 +460,7 @@ static const char* header_check_txt(const unsigned char *buffer, const unsigned
 	if(txt_nl==1)
 	{
 	  //ecrit_rapport(">%s<\ndoc => %s\n",buffer_lower,ext);
-	  FREE(buffer_lower);
+	  free(buffer_lower);
 	  return ext;
 	}
       }
@@ -422,16 +474,16 @@ static const char* header_check_txt(const unsigned char *buffer, const unsigned
 	   find_in_mem(buffer,buffer+buffer_size,"8BIM",4)==NULL &&
 	   find_in_mem(buffer,buffer+buffer_size,"adobe",5)==NULL) ||
 	  /* Text should not be found in zip because of compression */
-	  (file_recovery->file_stat->file_hint==&file_hint_zip))
+	  (file_recovery->file_stat->file_hint==&file_hint_zip && find_in_mem(buffer,buffer+buffer_size,zip_header,4)==NULL))
       {
-	FREE(buffer_lower);
+	free(buffer_lower);
 	return ext;
       }
-      FREE(buffer_lower);
+      free(buffer_lower);
       return NULL;
     }
 //    ecrit_rapport("ext=%s\n",ext);
-    FREE(buffer_lower);
+    free(buffer_lower);
     return ext;
   }
 }
@@ -439,7 +491,7 @@ static const char* header_check_txt(const unsigned char *buffer, const unsigned
 static int data_check_txt(const unsigned char *buffer, const unsigned int buffer_size, t_file_recovery *file_recovery)
 {
   unsigned int i;
-  unsigned char *buffer_lower=MALLOC(buffer_size+1);
+  unsigned char *buffer_lower=MALLOC(buffer_size+16);
   i=UTF2Lat(buffer_lower,&buffer[buffer_size/2],buffer_size/2);
   if(i<buffer_size/2)
   {
@@ -452,7 +504,9 @@ static int data_check_txt(const unsigned char *buffer, const unsigned int buffer
     }
     else if(i>=10)
       file_recovery->file_size+=i;
+    free(buffer_lower);
     return 2;
   }
+  free(buffer_lower);
   return 1;
 }
