@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -259,6 +259,10 @@ void change_geometry(t_param_disk *disk_car, char ** current_cmd)
       disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
   }
   disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+#ifdef __APPLE__
+  /* On MacOSX if HD contains some bad sectors, the disk size may not be correctly detected */
+  disk_car->disk_real_size=disk_car->disk_size;
+#endif
   ecrit_rapport("New geometry\n%s sector_size=%u\n", disk_car->description(disk_car), disk_car->sector_size);
 }
 
