@@ -2,7 +2,7 @@
 
     File: godmode.c
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -154,7 +154,7 @@ static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
   mvwaddstr(stdscr,10,0,msg_PART_HEADER);
   wmove(stdscr,22,0);
   wattrset(stdscr,A_STANDOUT);
-  wdoprintf(stdscr,"[  Quit  ]");
+  wdoprintf(stdscr,"[ Continue ]");
   wattroff(stdscr,A_STANDOUT);
   do
   {
@@ -311,9 +311,15 @@ static void warning_geometry(t_param_disk *disk_car, const unsigned int recomman
   wdoprintf(stdscr,"but the correct value may be %u.",recommanded_heads_per_cylinder);
   wmove(stdscr,8,0);
   wdoprintf(stdscr,"You can use the Geometry menu to change this value.");
+  wmove(stdscr,9,0);
+  wdoprintf(stdscr,"It's something to try if");
+  wmove(stdscr,10,0);
+  wdoprintf(stdscr,"- some partitions are not found by TestDisk");
+  wmove(stdscr,11,0);
+  wdoprintf(stdscr,"- or the partition table can not be written because partitions overlaps.");
   wmove(stdscr,22,0);
   wattrset(stdscr,A_STANDOUT);
-  wdoprintf(stdscr,"[  Quit  ]");
+  wdoprintf(stdscr,"[ Continue ]");
   wattroff(stdscr,A_STANDOUT);
   wrefresh(stdscr);
   if(*current_cmd!=NULL)
@@ -354,28 +360,49 @@ static unsigned int tab_insert(uint64_t *tab, uint64_t offset, unsigned int tab_
   return tab_nbr;
 }
 
+/*
+   Intel
+   - Display CHS
+   - Align: following configuration
+   - MinPartOffset: 512
+   - Geometry: care
+   Mac
+   - Display S
+   - Align to 4k (not required)
+   - MinPartOffset: 512
+   - Geometry: don't care
+   None
+   - Display S
+   - Align: none
+   - MinPartOffset: 0
+   - Geometry: don't care
+   Sun
+   - Display C
+   - Align to C boundaries
+   - MinPartOffset: 512
+   - Partition need to have H=0, S=1
+   - Geometry: required
+   XBox
+   - Display S
+   - Align: none
+   - MinPartOffset: 0x800
+   - Geometry: don't care
+*/
+
 t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump_ind, const int fast_mode, const int interface)
 {
+  unsigned char *buffer_disk;
+  /* TODO use circular buffer for try_offset and try_offset_raid */
   uint64_t try_offset[MAX_SEARCH_LOCATION];
   uint64_t try_offset_raid[MAX_SEARCH_LOCATION];
   unsigned int try_offset_nbr=0;
   unsigned int try_offset_raid_nbr=0;
-  unsigned int old_permil=0;
   int ind_stop=0;
   t_CHS start;
   t_list_part *list_part=NULL;
   t_list_part *list_part_bad=NULL;
   t_partition *partition=partition_new();
-#ifdef NEWBIE
-  t_param_disk *disk_car_geo16=(t_param_disk*)MALLOC(sizeof(*disk_car_geo16));
-  t_param_disk *disk_car_geo255=(t_param_disk*)MALLOC(sizeof(*disk_car_geo255));
-  disk_car_geo16->sector_size=disk_car->sector_size;
-  disk_car_geo16->CHS.sector=disk_car->CHS.sector;
-  disk_car_geo16->CHS.head=16-1;
-  disk_car_geo255->sector_size=disk_car->sector_size;
-  disk_car_geo255->CHS.sector=disk_car->CHS.sector;
-  disk_car_geo255->CHS.head=255-1;
-#endif
+  buffer_disk=(unsigned char*)MALLOC(16*DEFAULT_SECTOR_SIZE);
 
   if(interface!=0)
   {
@@ -388,42 +415,26 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
   ecrit_rapport("\nsearch_part()\n");
   ecrit_rapport("%s\n",disk_car->description(disk_car));
   start.head=(disk_car->arch==&arch_none?0:1);
-  for(start.cylinder=0;(start.cylinder<=disk_car->CHS.cylinder)&&(ind_stop==0);start.cylinder++,start.head=0)
+  for(start.cylinder=0;(start.cylinder<=disk_car->CHS.cylinder)&&(ind_stop==0);(start.cylinder%8==0 && disk_car->arch==&arch_mac?start.cylinder+=8:start.cylinder++),start.head=0)
   {
-    if(interface!=0)
+    if(interface!=0 && (disk_car->arch!=&arch_mac || (start.cylinder & 0xFFF)==0))
     {
-      unsigned int permil=start.cylinder*1000/(disk_car->CHS.cylinder+1);
-      if(permil>old_permil || disk_car->arch!=&arch_mac)
-      {
-	wmove(stdscr,ANALYSE_Y,ANALYSE_X);
-	wclrtoeol(stdscr);
-	wdoprintf(stdscr,"Analyse cylinder %5u/%u: %02u%%",start.cylinder,disk_car->CHS.cylinder,start.cylinder*100/(disk_car->CHS.cylinder+1));
-	wrefresh(stdscr);
-	old_permil=permil;
-      }
+      wmove(stdscr,ANALYSE_Y,ANALYSE_X);
+      wclrtoeol(stdscr);
+      wdoprintf(stdscr,"Analyse cylinder %5u/%u: %02u%%",start.cylinder,disk_car->CHS.cylinder,(unsigned int)((uint64_t)start.cylinder*100/(disk_car->CHS.cylinder+1)));
+      wrefresh(stdscr);
+      ind_stop|=check_enter_or_s(stdscr);
     }
     for(;(start.head<=disk_car->CHS.head)&&(ind_stop==0);start.head++)
     {
       unsigned int sector_inc=1;
       int test_nbr=0;
-      if(interface!=0)
-      {
-	ind_stop|=check_enter_or_s(stdscr);
-      }
       for(start.sector=1;start.sector<=disk_car->CHS.sector;start.sector+=sector_inc)
       {
 	int res=0;
 	unsigned int i;
-#ifdef NEWBIE
-	t_CHS start_geo16;
-	t_CHS start_geo255;
-#endif
 	partition->part_size=(uint64_t)0;
 	partition->part_offset=CHS2offset_inline(disk_car,&start);
-#ifdef NEWBIE
-	offset2CHS_inline(disk_car_geo16,partition->part_offset,&start_geo16);
-	offset2CHS_inline(disk_car_geo255,partition->part_offset,&start_geo255);
-#endif
 	if(test_nbr==0)
 	{
 	  int search_now=0;
@@ -438,45 +449,40 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
 	      try_offset[j]=try_offset[j+1];
 	    try_offset_nbr--;
 	  }
-	  search_now|= (start.sector==1 && fast_mode>1) || (start.sector==1 && start.head<=2)
-#ifdef NEWBIE
-	    || (start_geo16.sector==1 && fast_mode>1) || (start_geo16.sector==1 && start_geo16.head<=2)
-	    || (start_geo255.sector==1 && fast_mode>1) || (start_geo255.sector==1 && start_geo255.head<=2)
-#endif
-	    ;
+	  search_now|= (start.sector==1 && fast_mode>1) || (start.sector==1 && start.head<=2);
 	  if(search_now==0)
 	    test_nbr=6;
 	  else
 	  {
-	    res=search_type_64(disk_car,partition,debug,dump_ind);
+	    res=search_type_64(buffer_disk,disk_car,partition,debug,dump_ind);
 	    test_nbr++;
 	  }
 	}
 	if(res<=0 && test_nbr==1)
 	{
-	  res=search_type_2(disk_car,partition,debug,dump_ind);
+	  res=search_type_2(buffer_disk, disk_car,partition,debug,dump_ind);
 	  test_nbr++;
 	}
 	if(res<=0 && test_nbr==2)
 	{
-	    res=search_type_1(disk_car,partition,debug,dump_ind);
+	    res=search_type_1(buffer_disk,disk_car,partition,debug,dump_ind);
 	    test_nbr++;
 	}
 	if(res<=0 && test_nbr==3)
 	{
 	    /* Try to catch disklabel before BSD FFS partition */
-	    res=search_type_128(disk_car,partition,debug,dump_ind);
+	    res=search_type_128(buffer_disk,disk_car,partition,debug,dump_ind);
 	    test_nbr++;
 	}
 	if(res<=0 && test_nbr==4)
 	{
 	    /* Try to catch disklabel before BSD FFS partition */
-	    res=search_type_16(disk_car,partition,debug,dump_ind);
+	    res=search_type_16(buffer_disk,disk_car,partition,debug,dump_ind);
 	    test_nbr++;
 	}
 	if(res<=0 && test_nbr==5)
 	{
-	  res=search_type_0(disk_car,partition,debug,dump_ind);
+	  res=search_type_0(buffer_disk,disk_car,partition,debug,dump_ind);
 	  test_nbr++;
 	}
 	if(res<=0 && test_nbr==6)
@@ -485,36 +491,21 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
 	    test_nbr=10;
 	  else
 	  {
-	    if((start.sector==7 && (start.head<=2 || fast_mode>1))
-#ifdef NEWBIE
-		|| (start_geo16.sector==7 && (start_geo16.head<=2 || fast_mode>1))
-		|| (start_geo255.sector==7 && (start_geo255.head<=2 || fast_mode>1))
-#endif
-	      )
-	      res=search_FAT_backup(disk_car,partition,debug,dump_ind);
+	    if((start.sector==7 && (start.head<=2 || fast_mode>1)))
+	      res=search_FAT_backup(buffer_disk,disk_car,partition,debug,dump_ind);
 	    test_nbr++;
 	  }
 	}
 	if(res<=0 && test_nbr==7)
 	{
-	  if((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1))
-#ifdef NEWBIE
-	    || (start_geo16.sector==disk_car->CHS.sector && (start_geo16.head==disk_car->CHS.head || fast_mode>1))
-	    || (start_geo255.sector==disk_car->CHS.sector && (start_geo255.head==disk_car->CHS.head || fast_mode>1))
-#endif
-	    )
-	    res=search_NTFS_backup(disk_car,partition,debug,dump_ind);
+	  if((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1)))
+	    res=search_NTFS_backup(buffer_disk,disk_car,partition,debug,dump_ind);
 	  test_nbr++;
 	}
 	if(res<=0 && test_nbr==8)
 	{
-	  if((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1))
-#ifdef NEWBIE
-	    || (start_geo16.sector==disk_car->CHS.sector && (start_geo16.head==disk_car->CHS.head || fast_mode>1))
-	    || (start_geo255.sector==disk_car->CHS.sector && (start_geo255.head==disk_car->CHS.head || fast_mode>1))
-#endif
-	    )
-	    res=search_HFS_backup(disk_car,partition,debug,dump_ind);
+	  if((start.sector==disk_car->CHS.sector && (start.head==disk_car->CHS.head || fast_mode>1)))
+	    res=search_HFS_backup(buffer_disk,disk_car,partition,debug,dump_ind);
 	  test_nbr++;
 	}
 	if(res<=0 && test_nbr==9)
@@ -526,7 +517,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
 	  /* Block size: 1024, 2048 or 4096 bytes (8192 bytes on Alpha systems) */
 	  /* From e2fsprogs-1.34/lib/ext2fs/initialize.c: set_field(s_first_data_block, super->s_log_block_size ? 0 : 1); */
 	  /* Assumes that TestDisk is not running under Alpha and s_blocks_per_group=8 * block size */
-	  for(s_log_block_size=0;(s_log_block_size<=2)&&(res<1);s_log_block_size++)
+	  for(s_log_block_size=0;(s_log_block_size<=2)&&(res<=0);s_log_block_size++)
 	  {
 	    /* sparse superblock feature: The groups chosen are 0, 1 and powers of 3, 5 and 7. */
 	    /* Checking group 3 */
@@ -534,26 +525,12 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
 	    if(ref_offset>=hd_offset)
 	    {
 	      t_CHS start_ext2;
-#ifdef NEWBIE
-	      t_CHS start_ext2_geo16;
-	      t_CHS start_ext2_geo255;
-#endif
 	      offset2CHS_inline(disk_car,ref_offset-hd_offset,&start_ext2);
-#ifdef NEWBIE
-	      offset2CHS_inline(disk_car_geo16,partition->part_offset,&start_ext2_geo16);
-	      offset2CHS_inline(disk_car_geo255,partition->part_offset,&start_ext2_geo255);
-#endif
-	      if((start_ext2.sector==1 &&  (start_ext2.head<=2 || fast_mode>1))
-#ifdef NEWBIE
-		|| (start_ext2_geo16.sector==1 &&  (start_ext2_geo16.head<=2 || fast_mode>1))
-		|| (start_ext2_geo255.sector==1 &&  (start_ext2_geo255.head<=2 || fast_mode>1))
-#endif
-		)
+	      if((start_ext2.sector==1 &&  (start_ext2.head<=2 || fast_mode>1)))
 	      {
-		unsigned char buffer[2*DEFAULT_SECTOR_SIZE];
-		if(disk_car->read(disk_car,sizeof(buffer), &buffer, ref_offset)==0)
+		if(disk_car->read(disk_car,1024, buffer_disk, ref_offset)==0)
 		{
-		  const struct ext2_super_block *sb=(const struct ext2_super_block*)&buffer;
+		  const struct ext2_super_block *sb=(const struct ext2_super_block*)buffer_disk;
 		  if(le16(sb->s_block_group_nr)>0)
 		  {
 		    if(recover_EXT2(disk_car,sb,partition,debug,dump_ind)==0)
@@ -583,14 +560,13 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
 	  }
 	  if(fast_mode>1 || search_now>0)
 	  { /* Search Linux software RAID */
-	    unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-	    if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+	    if(disk_car->read(disk_car,8*DEFAULT_SECTOR_SIZE, buffer_disk, partition->part_offset)!=0)
 	    {
 	      res=-1;
 	    }
 	    else
 	    {
-	      if(recover_MD(disk_car,(const struct mdp_superblock_s*)&buffer,partition,debug,dump_ind)==0)
+	      if(recover_MD(disk_car,(const struct mdp_superblock_s*)buffer_disk,partition,debug,dump_ind)==0)
 		res=1;
 	      else
 		res=0;
@@ -637,10 +613,8 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
 	      if(align>0)
 	      {
 		end.sector=disk_car->CHS.sector;
-	      }
-	      if(align>1)
-	      {
-		end.head=disk_car->CHS.head;
+		if(align>1)
+		  end.head=disk_car->CHS.head;
 	      }
 	      help_factor_max=((uint64_t)CHS2offset_inline(disk_car, &end)-partition->part_offset+disk_car->sector_size-partition->part_size)/MD_RESERVED_BYTES;
 	      if(help_factor_max<3)
@@ -714,7 +688,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
       }
     }
   }
-  FREE(partition);
+  free(partition);
   if(ind_stop>0)
     ecrit_rapport("Search for partition aborted\n");
   if(list_part_bad!=NULL)
@@ -724,6 +698,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int debug, const int dump
       interface_part_bad(disk_car,list_part_bad);
   }
   delete_list_part(list_part_bad);
+  free(buffer_disk);
   return list_part;
 }
 
@@ -855,8 +830,8 @@ static t_list_part *reduce_structure(t_list_part *list_part)
 	prev->next=next;
       if(next!=NULL)
 	next->prev=prev;
-      FREE(element->part);
-      FREE(element);
+      free(element->part);
+      free(element);
     }
     else
       prev=element;
@@ -889,8 +864,8 @@ static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_
       order=element->part->order;
       if(element==list_part)
 	list_part=next;
-      FREE(element->part);
-      FREE(element);
+      free(element->part);
+      free(element);
       element=next;
     }
     else
