@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,7 @@
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h> 	/* open */
 #endif
+#include <stdio.h>
 #include <errno.h>
 #include "types.h"
 #ifdef HAVE_LINUX_TYPES_H
@@ -458,7 +459,7 @@ static t_param_disk *hd_identify(const int debug, const unsigned int disk, const
       }
       else
 	data->mode_enh=0;
-      FREE(param_disk_enh);
+      free(param_disk_enh);
     }
     return disk_car;
   }
@@ -492,11 +493,11 @@ static int disk_read(t_param_disk *disk_car,const unsigned int count, void *nom_
     /* ecrit_rapport("disk_read handle unaligned data\n"); */
     if(disk_read(disk_car,(unsigned)(((count/disk_car->sector_size)+2)*disk_car->sector_size),buffer,hd_offset/disk_car->sector_size*disk_car->sector_size))
     {
-      FREE(buffer);
+      free(buffer);
       return -1;
     }
     memcpy(nom_buffer,buffer+(hd_offset%disk_car->sector_size),count);
-    FREE(buffer);
+    free(buffer);
     return 0;
   }
   else
@@ -576,7 +577,7 @@ static int disk_clean(t_param_disk *disk_car)
   if(disk_car->data!=NULL)
   {
     struct info_disk_struct *data=disk_car->data;
-    FREE(disk_car->data);
+    free(disk_car->data);
     disk_car->data=NULL;
   }
   return 0;
@@ -1177,10 +1178,10 @@ static int file_clean(t_param_disk *disk_car)
     close(data->handle);
     if(data->buffer!=NULL)
     {
-      FREE(data->buffer);
+      free(data->buffer);
       data->buffer=NULL;
     }
-    FREE(disk_car->data);
+    free(disk_car->data);
     disk_car->data=NULL;
   }
   return 0;
@@ -1191,7 +1192,8 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
   struct info_file_struct *data=disk_car->data;
   if(count%disk_car->sector_size!=0 || offset%disk_car->sector_size!=0
 #ifdef O_DIRECT
-      || ((data->mode&O_DIRECT)!=0 && nom_buffer!=data->buffer)
+//      || ((data->mode&O_DIRECT)!=0 && nom_buffer!=data->buffer)
+      || ((data->mode&O_DIRECT)!=0 && (((size_t)(nom_buffer)) && (disk_car->sector_size-1)!=0))
 #endif
     )
   {
@@ -1204,7 +1206,7 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
     {
       if(data->buffer!=NULL)
       {
-	FREE(data->buffer);
+	free(data->buffer);
 	data->buffer=NULL;
       }
       data->buffer_size*=2;
@@ -1231,6 +1233,9 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
       return -1;
     }
     memcpy(nom_buffer,(char*)data->buffer+(offset%disk_car->sector_size),count);
+#ifdef DEBUG
+    ecrit_rapport("memcpy\n");
+#endif
     return 0;
   }
   {
@@ -1276,6 +1281,14 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
     ret=read(data->handle, nom_buffer, count);
 #endif
 #endif
+    /* Test only */
+    /*
+    if(!(offset+count<=120*512 || offset>=(120+1)*512))
+    {
+      ecrit_rapport("Simulated read failure\n");
+      return -1;
+    }
+    */
     if(ret!=count)
     {
       if(offset+count <= disk_car->disk_size && offset+count <= disk_car->disk_real_size)
@@ -1311,7 +1324,7 @@ static int file_write(t_param_disk *disk_car,const unsigned int count, const voi
     }
     memcpy(buffer+(offset%disk_car->sector_size),nom_buffer,count);
     ret=file_write(disk_car,(unsigned)(((count/disk_car->sector_size)+2)*disk_car->sector_size),buffer,offset/disk_car->sector_size*disk_car->sector_size);
-    FREE(buffer);
+    free(buffer);
     return ret;
   }
   else
@@ -1376,8 +1389,10 @@ static void autoset_geometry(t_param_disk * disk_car, const unsigned char *buffe
   }
   /* Round up because file is often truncated. */
   disk_car->CHS.cylinder=(disk_car->disk_size/disk_car->sector_size+(uint64_t)disk_car->CHS.sector*(disk_car->CHS.head+1)-1)/disk_car->CHS.sector/(disk_car->CHS.head+1)-1;
+  /*
   if(disk_car->disk_size<(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size)
     disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
+   */
 }
 
 static void compute_device_size(t_param_disk *disk_car)
@@ -1412,7 +1427,7 @@ static void compute_device_size(t_param_disk *disk_car)
     min_offset+=disk_car->sector_size;
   disk_car->disk_size=min_offset;
   disk_car->disk_real_size=disk_car->disk_size;
-  FREE(buffer);
+  free(buffer);
 }
 
 t_param_disk *file_test_availability(const char *device, const int debug, const t_arch_fnct *arch, const int testdisk_mode)
@@ -1565,7 +1580,7 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
 	disk_car->disk_real_size=disk_car->disk_size;
 	autoset_geometry(disk_car,buffer,debug);
       }
-      FREE(real_buffer);
+      free(real_buffer);
     }
     if(disk_car!=NULL)
     {
@@ -1598,8 +1613,9 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
       if(disk_car->disk_size==0)
       {
 	ecrit_rapport("Warning: can't get size for %s\n",device);
-	FREE(data);
-	FREE(disk_car);
+	free(data);
+	free(disk_car->device);
+	free(disk_car);
 	close(hd_h);
 	return NULL;
       }
@@ -1607,14 +1623,40 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
     }
     close(hd_h);
   }
-  return NULL;
+#ifdef DJGPP
+  {
+   int   dos_nr=0;
+   /* Check for device name. It must be like /dev/sdaXX and *
+    * XX is a value between 128 and 135 (0x80 to 0x88)      */
+   if(strcmp(device, "/dev/sda128") == 0)
+     dos_nr = 0x80;
+   if(strcmp(device, "/dev/sda129") == 0)
+     dos_nr = 0x81;
+   if(strcmp(device, "/dev/sda130") == 0)
+     dos_nr = 0x82;
+   if(strcmp(device, "/dev/sda131") == 0)
+     dos_nr = 0x83;
+   if(strcmp(device, "/dev/sda132") == 0)
+     dos_nr = 0x84;
+   if(strcmp(device, "/dev/sda133") == 0)
+     dos_nr = 0x85;
+   if(strcmp(device, "/dev/sda134") == 0)
+     dos_nr = 0x86;
+   if(strcmp(device, "/dev/sda135") == 0)
+     dos_nr = 0x87;
+   if(dos_nr>0)
+     disk_car = hd_identify(debug, dos_nr, arch, testdisk_mode);
+  }
+#endif
+  return disk_car;
 }
 
 void hd_update_geometry(t_param_disk *disk_car, const int allow_partial_last_cylinder, const int debug)
 {
-  unsigned char *buffer=MALLOC(disk_car->sector_size);
+  unsigned char *buffer;
   uint64_t pos;
   t_CHS pos_CHS;
+  buffer=MALLOC(disk_car->sector_size);
   if(disk_car->autodetect!=0 || disk_car->arch==&arch_mac)
   {
     if(disk_car->read(disk_car,disk_car->sector_size, buffer, 0)==0)
@@ -1659,7 +1701,7 @@ void hd_update_geometry(t_param_disk *disk_car, const int allow_partial_last_cyl
     data->CHSR.cylinder=disk_car->CHS.cylinder;
   }
 #endif
-  FREE(buffer);
+  free(buffer);
 }
 
 void hd_update_all_geometry(const t_list_disk * list_disk, const int allow_partial_last_cylinder, const int debug)
@@ -1782,7 +1824,7 @@ t_param_disk *file_test_availability_win32(const char *device, const int debug,
     {
       /* if(disk_car->sector_size!=4096) */
       {
-	FREE(disk_car);
+	free(disk_car);
 	CloseHandle(handle);
 	return NULL;
       }
@@ -1854,8 +1896,9 @@ t_param_disk *file_test_availability_win32(const char *device, const int debug,
     if(disk_car->disk_size==0)
     {
       ecrit_rapport("Warning: can't get size for %s\n",device);
-      FREE(data);
-      FREE(disk_car);
+      free(data);
+      free(disk_car->device);
+      free(disk_car);
       CloseHandle(handle);
       return NULL;
     }
@@ -1892,10 +1935,10 @@ static int file_win32_clean(t_param_disk *disk_car)
     CloseHandle(data->handle);
     if(data->buffer!=NULL)
     {
-      FREE(data->buffer);
+      free(data->buffer);
       data->buffer=NULL;
     }
-    FREE(disk_car->data);
+    free(disk_car->data);
     disk_car->data=NULL;
   }
   return 0;
@@ -1912,11 +1955,11 @@ static unsigned int file_win32_compute_sector_size(HANDLE handle, const unsigned
     ret=ReadFile(handle, buffer,sector_size,&dwByteRead,NULL);
     if(ret && dwByteRead==sector_size)
     {
-      FREE(buffer);
+      free(buffer);
       return sector_size;
     }
   }
-  FREE(buffer);
+  free(buffer);
   return sector_size_default;
 }
 
@@ -1938,7 +1981,7 @@ static int file_win32_read(t_param_disk *disk_car,const unsigned int count, void
     {
       if(data->buffer!=NULL)
       {
-	FREE(data->buffer);
+	free(data->buffer);
 	data->buffer=NULL;
       }
       data->buffer_size*=2;
