@@ -15,52 +15,65 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
-#define MAX_LINES	200
-#define LINE_LENGTH 80
-#define BUFFER_LINE_LENGTH 4*LINE_LENGTH
-#define MAXIMUM_PARTS 60
-#define COL_ID_WIDTH 25
-#define WARNING_START 23
-#define COLUMNS 80
+#ifdef HAVE_NCURSES_H
+#include <ncurses.h>
+#elif defined(HAVE_NCURSES_NCURSES_H)
+#include <ncurses/ncurses.h>
+#elif defined(HAVE_NCURSES_CURSES_H)
+#include <ncurses/curses.h>
+#elif defined(HAVE_CURSES_H)
+#include <curses.h>
+#endif
+
+#include "dir.h"
+
+#define MAX_LINES		200
+#define LINE_LENGTH 		80
+#define BUFFER_LINE_LENGTH 	4*LINE_LENGTH
+#define MAXIMUM_PARTS 		60
+#define COL_ID_WIDTH 		25
+#define WARNING_START 		23
+#define COLUMNS 		80
 
 
-#define COMMAND_LINE_X	0
-#define COMMAND_LINE_Y  18
-#define DUMP_X	0
-#define DUMP_Y	5 + 2
-#define DUMP_MAX_LINES	14
-#define INTER_DUMP_X	DUMP_X
-#define INTER_DUMP_Y	DUMP_Y+DUMP_MAX_LINES+1
-#define INTER_ANALYSE_X	0
-#define INTER_ANALYSE_Y 7
-#define INTER_MAX_LINES 13
-#define INTER_ANALYSE_MENU_X 0
-#define INTER_ANALYSE_MENU_Y 23
-#define INTER_OPTION_X  0
-#define INTER_OPTION_Y	10
-#define INTER_PARTITION_X  0
+#define COMMAND_LINE_X		0
+#define COMMAND_LINE_Y  	18
+#define DUMP_X			0
+#define DUMP_Y			5 + 2
+#define DUMP_MAX_LINES		14
+#define INTER_DUMP_X		DUMP_X
+#define INTER_DUMP_Y		DUMP_Y+DUMP_MAX_LINES+1
+#define INTER_ANALYSE_X		0
+#define INTER_ANALYSE_Y 	7
+#define INTER_MAX_LINES 	13
+#define INTER_ANALYSE_MENU_X 	0
+#define INTER_ANALYSE_MENU_Y 	23
+#define INTER_OPTION_X  	0
+#define INTER_OPTION_Y		10
+#define INTER_PARTITION_X  	0
 #define INTER_PARTITION_Y	8
-#define INTER_MAIN_X	0
-#define INTER_MAIN_Y	18
+#define INTER_MAIN_X		0
+#define INTER_MAIN_Y		18
 /* Constants for menuType parameter of menuSelect function */
-#define MENU_HORIZ 1
-#define MENU_VERT 2
-#define MENU_ACCEPT_OTHERS 4
-#define MENU_BUTTON 8
-#define MENU_VERT_WARN	16
+#define MENU_HORIZ 		1
+#define MENU_VERT 		2
+#define MENU_ACCEPT_OTHERS 	4
+#define MENU_BUTTON 		8
+#define MENU_VERT_WARN		16
+#define MENU_VERT_ARROW2VALID	32
 /* Miscellenous constants */
-#define MENU_SPACING 2
-#define MENU_MAX_ITEMS 256 /* for simpleMenu function */
-#define key_CR '\015'
-#define key_ESC '\033'
-#define key_DEL '\177'
-#define key_BELL '\007'
+#define MENU_SPACING 		2
+#define MENU_MAX_ITEMS 		256 /* for simpleMenu function */
+#define key_CR 			'\015'
+#define key_ESC 		'\033'
+#define key_DEL 		'\177'
+#define key_BELL 		'\007'
 /* '\014' == ^L */
-#define key_REDRAWKEY '\014'
+#define key_REDRAWKEY 		'\014'
 
 
 enum buffer_cmd {BUFFER_RESET, BUFFER_ADD, BUFFER_WRITE,BUFFER_SHOW};
@@ -110,3 +123,9 @@ void screen_buffer_to_log(void);
 int check_enter_or_s(WINDOW *window);
 int interface_partition_type(t_param_disk *disk_car, const int allow_partial_last_cylinder, const int debug, char**current_cmd);
 void intrf_no_disk(const char *prog_name);
+char *ask_log_location(const char*filename);
+int ask_log_creation(void);
+FILE* log_open(const char*default_filename, const char *mode, const char *prog_name, int argc, char**argv);
+char *ask_location(const char*msg, const char *src_dir);
+int aff_txt(int line, WINDOW *window, const char *_format, ...) __attribute__ ((format (printf, 3, 4)));
+
