@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -24,12 +24,12 @@
 #endif
  
 #include <stdarg.h>
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>	/* geteuid */
-#endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -56,12 +56,8 @@
 #include "chgtype.h"
 #include "edit.h"
 #include "savehdr.h"
-#include "dir.h"
-#include "ext2_dir.h"
+#include "dirpart.h"
 #include "fat.h"
-#include "fat_dir.h"
-#include "ntfs_dir.h"
-#include "rfs_dir.h"
 
 #define INTER_DISK_X	0
 #define INTER_DISK_Y	7
@@ -225,13 +221,6 @@ static int menu_disk(t_param_disk *disk_car, int debug,int dump_ind, const int s
       }
       else
       {
-	if(*current_cmd[0]!='\0')
-	{
-	  ecrit_rapport("error >%s<\n",*current_cmd);
-	  while(*current_cmd[0]!='\0')
-	    (*current_cmd)++;
-	  return 1;
-	}
 	command='Q';
       }
     }
@@ -284,10 +273,9 @@ static int menu_disk(t_param_disk *disk_car, int debug,int dump_ind, const int s
   return 0;
 }
 
-int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, const int saveheader, const char *real_prog_name, const char *cmd_device, char *cmd_run)
+int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, const int saveheader, const char *cmd_device, char **current_cmd)
 {
-  int command;
-  int real_key;
+  int command='Q';
   int allow_partial_last_cylinder=0;
   int done=0;
   int menu=0;
@@ -303,7 +291,6 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
     { 'Q',"Quit","Quit program"},
     { 0,NULL,NULL}
   };
-  char *current_cmd=cmd_run;
   if(cmd_device!=NULL)
   {
     for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
@@ -314,8 +301,6 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
   }
   else
     current_disk=list_disk;
-  if(start_ncurses("TestDisk",real_prog_name))
-    return 1;
   if(current_disk==NULL)
   {
     intrf_no_disk("TestDisk");
@@ -350,24 +335,52 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
       options="OQ";
     else
       options="PNOQ";
-    wmove(stdscr,20,0);
-    wdoprintf(stdscr,"Note: Disk capacity must be correctly detected for a successful recovery.");
-    wmove(stdscr,21,0);
-    wdoprintf(stdscr,"If a disk listed above has incorrect size, check HD jumper settings, BIOS");
-    wmove(stdscr,22,0);
-    wdoprintf(stdscr,"detection, and install the latest OS patches and disk drivers."); 
-    if(current_cmd!=NULL)
     {
-      while(current_cmd[0]==',')
-	current_cmd++;
-      if(current_cmd[0]=='\0')
-	command='Q';
+      int line=20;
+#if defined(__CYGWIN__) || defined(__MINGW32__)
+#else
+#ifndef DJGPP
+#ifdef HAVE_GETEUID
+      if(geteuid()!=0)
+      {
+	wmove(stdscr,line++,0);
+	wdoprintf(stdscr,"Note: Some disks won't appear unless you're root user.");
+      }
+#endif
+#endif
+#endif
+      wmove(stdscr,line++,0);
+      if(line==22)
+	wdoprintf(stdscr,"Disk capacity must be correctly detected for a successful recovery.");
       else
-	command='O';
+	wdoprintf(stdscr,"Note: Disk capacity must be correctly detected for a successful recovery.");
+      wmove(stdscr,line++,0);
+      wdoprintf(stdscr,"If a disk listed above has incorrect size, check HD jumper settings, BIOS");
+      wmove(stdscr,line++,0);
+      wdoprintf(stdscr,"detection, and install the latest OS patches and disk drivers."); 
     }
-    else
+    if(*current_cmd!=NULL)
+    {
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(*current_cmd[0]=='\0')
+	command='Q';
+      else if(strncmp(*current_cmd,"inter",4)==0)
+      {
+	cmd_device=NULL;
+	*current_cmd=NULL;
+      }
+      else
+      {
+	if(command=='O')
+	  command='Q';
+	else
+	  command='O';
+      }
+    }
+    if(*current_cmd==NULL)
       command = wmenuSelect_ext(stdscr,INTER_MAIN_Y, INTER_MAIN_X, menuMain, 8,
-	    options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, &menu,&real_key);
+	    options, MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, &menu,NULL);
     switch(command)
     {
       case KEY_UP:
@@ -392,9 +405,9 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
 	break;
       case 'o':
       case 'O':
-	if(interface_partition_type(current_disk->disk, allow_partial_last_cylinder, debug, &current_cmd)==0)
+	if(interface_partition_type(current_disk->disk, allow_partial_last_cylinder, debug, current_cmd)==0)
 	{
-	  if(menu_disk(current_disk->disk, debug, dump_ind, saveheader, &current_cmd))
+	  if(menu_disk(current_disk->disk, debug, dump_ind, saveheader, current_cmd))
 	    done=1;
 	  menu=3; /* select quit */
 	}
@@ -405,8 +418,6 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
 	break;
     }
   }
-  end_ncurses();
-  ecrit_rapport("\n");
   return 0;
 }
 
@@ -419,7 +430,7 @@ void interface_analyse(t_param_disk *disk_car, const int debug, const int savehe
     { 'P', "Previous",""},
     { 'N', "Next","" },
     { 'Q',"Proceed","Try to locate partition"},
-    { 'S', "Save","Save partition structure"},
+    { 'B', "Backup","Save current partition list to backup.log file and proceed"},
     { 0, NULL, NULL }
   };
   ecrit_rapport("\nAnalyse ");
@@ -449,16 +460,19 @@ void interface_analyse(t_param_disk *disk_car, const int debug, const int savehe
     {
       (*current_cmd)+=6;
       if(list_part!=NULL)
-	command='S';
+	command='B';
     }
     screen_buffer_to_log();
   }
   else
-    command=screen_buffer_display(stdscr,(list_part!=NULL?"QS":"Q"),menuAnalyse);
-  if(command=='S')
+    command=screen_buffer_display(stdscr,(list_part!=NULL?"QB":"Q"),menuAnalyse);
+  if(command=='B')
   {
     ecrit_rapport("Backup partition structure\n");
-    partition_save(disk_car,list_part,debug);
+    if(partition_save(disk_car,list_part,debug)<0)
+    {
+      display_message("Can't create backup.log.\n");
+    }
   }
   delete_list_part(list_part);
 }
@@ -575,7 +589,7 @@ static void interface_options(int *dump_ind, int *align, int *allow_partial_last
       int real_key;
       struct MenuItem menuOptions[]=
       {
-	{ 'E',NULL,"Expert mode adds some functionaliteis"},
+	{ 'E',NULL,"Expert mode adds some functionalities"},
 	{ 'C',NULL,"Partitions are aligned on cylinder/head boundaries" },
 	{ 'A',NULL,""},
 	{ 'D',NULL,"Dump essential sectors" },
@@ -599,7 +613,7 @@ static void interface_options(int *dump_ind, int *align, int *allow_partial_last
       menuOptions[3].name=*dump_ind?"Dump : Yes":"Dump : No";
       aff_copy(stdscr);
 
-      car=wmenuSelect_ext(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "ECADQ", MENU_VERT, &menu,&real_key);
+      car=wmenuSelect_ext(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "ECADQ", MENU_VERT|MENU_VERT_ARROW2VALID, &menu,&real_key);
       switch(car)
       {
 	case 'd':
@@ -917,38 +931,7 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       case 'p':
       case 'P':
 	if(list_part!=NULL)
-	{
-	  WINDOW *window=newwin(0,0,0,0);	/* full screen */
-	  aff_copy(window);
-	  ecrit_rapport("\n");
-	  switch(pos->part->upart_type)
-	  {
-	    case UP_FAT12:
-	    case UP_FAT16:
-	    case UP_FAT32:
-	      dir_partition_fat(window,disk_car,pos->part,debug,current_cmd);
-	      break;
-	    case UP_EXT2:
-	    case UP_EXT3:
-	      dir_partition_ext2(window,disk_car,pos->part,debug,current_cmd);
-	      break;
-	    case UP_RFS:
-	    case UP_RFS2:
-	    case UP_RFS3:
-	      dir_partition_reiser(window,disk_car,pos->part,debug,current_cmd);
-	      break;
-	    case UP_NTFS:
-	      dir_partition_ntfs(window,disk_car,pos->part,debug,current_cmd);
-	      break;
-	    default:
-	      break;
-	  }
-	  delwin(window);
-	  (void) clearok(stdscr, TRUE);
-#ifdef HAVE_TOUCHWIN
-	  touchwin(stdscr);
-#endif
-	}
+	  dir_partition(disk_car,pos->part,debug,current_cmd);
 	break;
       case 'l':
       case 'L':
@@ -980,13 +963,15 @@ static t_list_part *backup_merge(t_list_part *list_part,t_list_part *backup_part
 
 static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part, const int debug)
 {
-  t_backup_disk *list_backup=partition_load(disk_car,debug);
+  struct list_head *backup_walker=NULL;
+  t_backup_disk *backup_list=partition_load(disk_car,debug);
   {
-    t_backup_disk *backup;
     ecrit_rapport("interface_load\n");
-    for(backup=list_backup;backup!=NULL;backup=backup->next)
+    list_for_each(backup_walker,&backup_list->list)
     {
       t_list_part *element;
+      t_backup_disk *backup;
+      backup=list_entry(backup_walker, t_backup_disk, list);
       ecrit_rapport("%s %s",backup->description,ctime(&backup->my_time));
       for(element=backup->list_part;element!=NULL;element=element->next)
 	aff_part_rapport(disk_car,element->part);
@@ -995,10 +980,10 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
   {
     int quit=0;
     int offset=0;
-    int pos_num=0;
-    t_backup_disk *pos=list_backup;
+    int backup_current_num=0;
     int rewrite=1;
     int menu=3;   /* default : quit */
+    struct list_head *backup_current=backup_list->list.next;
     struct MenuItem menuLoadBackup[]=
     {
       { 'P', "Previous",""},
@@ -1011,12 +996,11 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
     do
     {
       int i;
-      t_backup_disk *backup;
       if(rewrite)
       {
 	aff_copy(stdscr);
 	mvwaddstr(stdscr,4,0,disk_car->description(disk_car));
-	if(list_backup!=NULL)
+	if(backup_list!=NULL)
 	{
 	  mvwaddstr(stdscr,5,0,"Choose the backup you want to restore:");
 	  mvwaddstr(stdscr,20,0,"PS: Don't worry you will have to confirm the partition restoration.");
@@ -1027,14 +1011,16 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
 	}
 	rewrite=0;
       }
-      if(list_backup!=NULL)
+      if(backup_list!=NULL)
       {
-	for(i=0,backup=list_backup;(backup!=NULL) && (i<offset);backup=backup->next,i++);
-	for(i=offset;(backup!=NULL) &&((i-offset)<INTER_STRUCTURE);i++,backup=backup->next)
+	t_backup_disk *backup=NULL;
+	for(i=0,backup_walker=backup_list->list.next;(backup_walker!=&backup_list->list) && (i<offset);backup_walker=backup_walker->next,i++);
+	for(i=offset;(backup_walker!=&backup_list->list) &&((i-offset)<INTER_STRUCTURE);i++,backup_walker=backup_walker->next)
 	{
+	  backup=list_entry(backup_walker, t_backup_disk, list);
 	  wmove(stdscr,8+i-offset,0);
 	  wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
-	  if(backup==pos)
+	  if(backup_walker==backup_current)
 	  {
 	    wattrset(stdscr,A_STANDOUT);
 	    wdoprintf(stdscr,"%s %s",backup->description,ctime(&backup->my_time));
@@ -1071,49 +1057,41 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
 	  quit=2;
 	  break;
 	case KEY_UP:
-	  if(list_backup!=NULL)
+	  if(backup_current->prev!=&backup_list->list)
 	  {
-	    if(pos->prev!=NULL)
-	    {
-	      pos=pos->prev;
-	      pos_num--;
-	    }
-	    if(pos_num<offset)
+	    backup_current=backup_current->prev;
+	    backup_current_num--;
+	    if(backup_current_num<offset)
 	      offset--;
 	  }
 	  break;
 	case KEY_DOWN:
-	  if(list_backup!=NULL)
+	  if(backup_current->next!=&backup_list->list)
 	  {
-	    if(pos->next!=NULL)
-	    {
-	      pos=pos->next;
-	      pos_num++;
-	    }
-	    if(pos_num>=offset+INTER_STRUCTURE)
+	    backup_current=backup_current->next;
+	    backup_current_num++;
+	    if(backup_current_num>=offset+INTER_STRUCTURE)
 	      offset++;
 	  }
 	  break;
 	case KEY_PPAGE:
-	  if(list_backup!=NULL)
 	  {
-	    for(i=0;(i<INTER_STRUCTURE) && (pos->prev!=NULL);i++)
+	    for(i=0;(i<INTER_STRUCTURE) && (backup_current->prev!=&backup_list->list);i++)
 	    {
-	      pos=pos->prev;
-	      pos_num--;
-	      if(pos_num<offset)
+	      backup_current=backup_current->prev;
+	      backup_current_num--;
+	      if(backup_current_num<offset)
 		offset--;
 	    }
 	  }
 	  break;
 	case KEY_NPAGE:
-	  if(list_backup!=NULL)
 	  {
-	    for(i=0;(i<INTER_STRUCTURE) && (pos->next!=NULL);i++)
+	    for(i=0;(i<INTER_STRUCTURE) && (backup_current->next!=&backup_list->list);i++)
 	    {
-	      pos=pos->next;
-	      pos_num++;
-	      if(pos_num>=offset+INTER_STRUCTURE)
+	      backup_current=backup_current->next;
+	      backup_current_num++;
+	      if(backup_current_num>=offset+INTER_STRUCTURE)
 		offset++;
 	    }
 	  }
@@ -1123,27 +1101,29 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
 	  break;
       }
     } while(quit==0);
-    if(pos!=NULL && quit==2)
+    if(backup_current!=NULL && quit==2)
     {
       t_list_part *partition;
-      for(partition=pos->list_part;partition!=NULL;partition=partition->next)
+      t_backup_disk *backup;
+      backup=list_entry(backup_walker, t_backup_disk, list);
+      for(partition=backup->list_part;partition!=NULL;partition=partition->next)
       {
 	/* Check partition and load partition name */
 	disk_car->arch->check_part(disk_car,debug,partition->part,0);
       }
-      list_part=backup_merge(list_part,pos->list_part,debug);
+      list_part=backup_merge(list_part,backup->list_part,debug);
     }
   }
   { /* Cleanup */
-    t_backup_disk *backup;
-    backup=list_backup;
-    while(backup!=NULL)
+    struct list_head *backup_walker_next = NULL;
+    list_for_each_safe(backup_walker,backup_walker_next,&backup_list->list)
     {
-      t_backup_disk *next=backup->next;
+      t_backup_disk *backup;
+      backup=list_entry(backup_walker, t_backup_disk, list);
       delete_list_part(backup->list_part);
-      FREE(backup);
-      backup=next;
+      free(backup);
     }
+    free(backup_list);
   }
   return list_part;
 }
