@@ -2,7 +2,7 @@
 
     File: intrface.h
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -25,7 +25,7 @@
 #define INTER_STRUCTURE	13
 #define INTER_BAD_PART	10
 
-int do_curses_testdisk(int debug, int dump_ind, const t_list_disk *list_disk, const int saveheader, const char *real_prog_name, const char *cmd_device, char *cmd_run);
+int do_curses_testdisk(int debug, int dump_ind, const t_list_disk *list_disk, const int saveheader, const char *cmd_device, char **current_cmd);
 int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_search_deeper, const int can_ask_minmax_ext, char **current_cmd, int *menu);
 
 t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd);
