@@ -2,7 +2,7 @@
 
     File: jfs.h
 
-    Copyright (C) 2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004,2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,10 +15,13 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+/* real size is 184 */
+#define JFS_SUPERBLOCK_SIZE 512
+
 #define L2BPERDMAP 13      /* l2 num of blks per dmap */
 int check_JFS(t_param_disk *disk_car,t_partition *partition,const int debug);
 int recover_JFS(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partition *partition,const int debug, const int dump_ind);
