@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -57,7 +57,6 @@
 #define msg_ONLY1MUSTBOOT "Only one partition must be bootable\n"
 #define msg_SAME_SPACE    "Space conflict between the following two partitions\n"
 #define msg_CHOICE        "\nChoice ? "
-#define msg_MALLOC_ERROR  "\nMemory allocation error"
 #define msg_FREE_ERROR    "\nFree memory allocation error"
 #define msg_WRITE_CLEAN_TABLE     "Clear MBR partition table by writing zero bytes to it? (Y/N) "
 #define msg_WRITE_MBR_CODE    "Write a new copy of MBR code to first sector? (Y/N) "
