@@ -15,14 +15,17 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
  
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -43,11 +46,18 @@ static int test_netware(t_param_disk *disk_car, const struct disk_netware *netwa
 
 int check_netware(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
-  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
-  { return 1; }
-  if(test_netware(disk_car,(const struct disk_netware *)&buffer,partition,debug,0)!=0)
+  unsigned char *buffer=MALLOC(DEFAULT_SECTOR_SIZE);
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, partition->part_offset)!=0)
+  {
+    free(buffer);
     return 1;
+  }
+  if(test_netware(disk_car,(const struct disk_netware *)buffer,partition,debug,0)!=0)
+  {
+    free(buffer);
+    return 1;
+  }
+  free(buffer);
   return 0;
 }
 
