@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -27,11 +27,14 @@
 #include <string.h>
 #endif
 #include <ctype.h>
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
+#include "dirpart.h"
 #include "ntfs.h"
-#include "ntfs_dir.h"
 #include "fnctdsk.h"
 #include "lang.h"
 #include "io_redir.h"
@@ -130,6 +133,7 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
     const char *options="DLQ";
     int do_write=0;
     int do_exit=0;
+    int no_confirm=0;
     int command;
     do
     {
@@ -171,6 +175,17 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
 	  (*current_cmd)+=4;
 	  command='D';
 	}
+	else if(strncmp(*current_cmd,"noconfirm",9)==0)
+	{
+	  command=0;	/* do nothing */
+	  no_confirm=1;
+	  (*current_cmd)+=9;
+	}
+      	else if(strncmp(*current_cmd,"write",5)==0)
+	{
+	  command='W';
+	  (*current_cmd)+=5;
+	}
       }
       else
 	command=wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuSaveBoot,8,options,MENU_HORIZ | MENU_BUTTON, 1);
@@ -205,7 +220,7 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
 	case 'l':
 	case 'L':
 	  io_redir_add_redir(disk_car,partition->part_offset,sizeof(newboot),0,newboot);
-	  dir_partition_ntfs(stdscr,disk_car, partition, 0,current_cmd);
+	  dir_partition(disk_car, partition, 0,current_cmd);
 	  io_redir_del_redir(disk_car,partition->part_offset);
 	  break;
 	case 'q':
@@ -214,7 +229,7 @@ static int create_ntfs_boot_sector(t_param_disk *disk_car, t_partition *partitio
 	  break;
       }
     } while(do_write==0 && do_exit==0);
-    if(do_write!=0 && ask_confirmation("Write new NTFS boot sector, confirm ? (Y/N)")!=0)
+    if(do_write!=0 && (no_confirm!=0 || ask_confirmation("Write new NTFS boot sector, confirm ? (Y/N)")!=0))
     {
       ecrit_rapport("Write new boot!\n");
       /* Write boot sector and backup boot sector */
@@ -312,6 +327,7 @@ static int read_mft_info(t_param_disk *disk_car, t_partition *partition, uint64_
     ecrit_rapport("ntfs_find_mft: mft_record_size     %u\n",*mft_record_size);
     ecrit_rapport("\n");
   }
+  *sectors_per_cluster=0;
   return 3;
 }
 
@@ -351,7 +367,8 @@ int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int de
 	  {
 	    ecrit_rapport("ntfs_find_mft: mft_lcn             %lu\n",(long unsigned int)mft_lcn);
 	    ecrit_rapport("ntfs_find_mft: mftmirr_lcn         %lu\n",(long unsigned int)mftmirr_lcn);
-	    ind_stop=TRUE;
+	    if(expert==0 || ask_confirmation("Use MFT from %lu, confirm ? (Y/N)",(long unsigned int)mft_lcn)!=0)
+	      ind_stop=TRUE;
 	  }
 	}
       }
@@ -386,7 +403,8 @@ int rebuild_NTFS_BS(t_param_disk *disk_car, t_partition *partition, const int de
 	  {
 	    ecrit_rapport("ntfs_find_mft: mft_lcn             %lu\n",(long unsigned int)mft_lcn);
 	    ecrit_rapport("ntfs_find_mft: mftmirr_lcn         %lu\n",(long unsigned int)mftmirr_lcn);
-	    ind_stop=TRUE;
+	    if(expert==0 || ask_confirmation("Use MFT from %lu, confirm ? (Y/N)",(long unsigned int)mft_lcn)!=0)
+	      ind_stop=TRUE;
 	  }
 	}
       }
