@@ -1,7 +1,7 @@
 /**
  * ntfs_dir.c - Part of the TestDisk project.
  *
- * Copyright (c) 2004-2005 Christophe Grenier
+ * Copyright (c) 2004-2006 Christophe Grenier
  *
  * Original version comes from the Linux-NTFS project.
  * Copyright (c) 2003 Lode Leroy
@@ -21,13 +21,12 @@
  * You should have received a copy of the GNU General Public License
  * along with this program (in the main directory of the Linux-NTFS
  * distribution in the file COPYING); if not, write to the Free Software
- * Foundation,Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation,Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
  
-#ifdef HAVE_LIBNTFS
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -48,26 +47,36 @@
 #include <machine/endian.h>
 #endif
 #include <stdarg.h>
+#include "types.h"
+
+#ifdef HAVE_LIBNTFS
 #include <ntfs/attrib.h>
 #ifdef HAVE_NTFS_VERSION_H
 #include <ntfs/version.h>
 #endif
+#endif
+
+#include "common.h"
+#include "intrf.h"
+#include "ntfs.h"
+#include "dir.h"
+#include "ntfs_dir.h"
+#include "io_redir.h"
+
+
+#ifdef HAVE_LIBNTFS
 #define MAX_PATH    1024
 #define PATH_SEP      '/'
 #define NTFS_DT_DIR               4
 #define NTFS_DT_REG             8
 #define NTFS_TIME_OFFSET ((s64)(369 * 365 + 89) * 24 * 3600 * 10000000)
-#include "types.h"
-#include "common.h"
-#include "intrf.h"
-#include "ntfs.h"
-#include "ntfs_dir.h"
-#include "dir.h"
-#include "io_redir.h"
+
 struct list_dir_struct {
 	t_file_data *dir_list;
 	t_file_data *current_file;
 	ntfs_volume *vol;
+	t_my_data *my_data;
+	unsigned long int inode;
 };
 
 /*
@@ -87,14 +96,14 @@ extern int ntfs_readdir(ntfs_inode *dir_ni, s64 *pos,
 extern u64 ntfs_inode_lookup_by_name(ntfs_inode *dir_ni,
                 const ntfschar *uname, const int uname_len);
 static time_t ntfs2utc (s64 ntfstime);
-static int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
+static int ntfs_list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type);
-static ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, const char *pathname);
 static t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster);
 static ATTR_RECORD * find_attribute(const ATTR_TYPES type, ntfs_attr_search_ctx *ctx);
 static ATTR_RECORD * find_first_attribute(const ATTR_TYPES type, MFT_RECORD *mft);
 static int ntfs_copy(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int first_inode);
+static void dir_partition_ntfs_close(t_dir_data *dir_data);
 
 /**
  * ntfs2utc - Convert an NTFS time to Unix time
@@ -111,10 +120,10 @@ static time_t ntfs2utc (s64 ntfstime)
 }
 
 /**
- * list_entry
+ * ntfs_list_entry
  * FIXME: Should we print errors as we go along? (AIA)
  */
-static int list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
+static int ntfs_list_entry(  struct list_dir_struct *ls, const ntfschar *name, 
 		const int name_len, const int name_type, const s64 pos,
 		const MFT_REF mref, const unsigned dt_type)
 {
@@ -123,7 +132,7 @@ static int list_entry(  struct list_dir_struct *ls, const ntfschar *name,
   filename = calloc (1, MAX_PATH);
   if (!filename)
   {
-    ecrit_rapport("list_entry calloc failed\n");
+    ecrit_rapport("ntfs_list_entry calloc failed\n");
     return -1;
   }
 
@@ -173,10 +182,6 @@ static int list_entry(  struct list_dir_struct *ls, const ntfschar *name,
 	    ctx->attr);
     }
 
-    /*		if (opts.inode) */
-    /*			printf("%7lld    %8lld %s %s\n", MREF(mref), filesize, */
-    /*					t_buf + 4, filename); */
-    /*		else */
     {
       t_file_data *new_file=MALLOC(sizeof(*new_file));
       memcpy(new_file->name,filename,(MAX_PATH<sizeof(new_file->name)?MAX_PATH:sizeof(new_file->name)));
@@ -185,7 +190,7 @@ static int list_entry(  struct list_dir_struct *ls, const ntfschar *name,
       new_file->filestat.st_dev=0;
       new_file->filestat.st_ino=MREF(mref);
       new_file->filestat.st_mode = (dt_type == NTFS_DT_DIR?LINUX_S_IFDIR| LINUX_S_IRUGO | LINUX_S_IXUGO:LINUX_S_IFREG | LINUX_S_IRUGO);
-      new_file->filestat.st_nlink=0;
+      new_file->filestat.st_nlink=1;
       new_file->filestat.st_uid=0;
       new_file->filestat.st_gid=0;
       new_file->filestat.st_rdev=0;
@@ -250,98 +255,6 @@ free:
  * soon as the function returns.
  */
 
-
-/**
- * utils_pathname_to_inode - Find the inode which represents the given pathname
- * @vol:       An ntfs volume obtained from ntfs_mount
- * @parent:    A directory inode to begin the search (may be NULL)
- * @pathname:  Pathname to be located
- *
- * Take an ASCII pathname and find the inode that represents it.  The function
- * splits the path and then descends the directory tree.  If @parent is NULL,
- * then the root directory '.' will be used as the base for the search.
- *
- * Return:  inode  Success, the pathname was valid
- *	    NULL   Error, the pathname was invalid, or some other error occurred
- */
-static ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, const char *pathname)
-{
-	u64 inum;
-	int len;
-	char *p, *q;
-	ntfs_inode *ni;
-	ntfs_inode *result  = NULL;
-	ntfschar    *unicode = NULL;
-	char       *ascii   = NULL;
-
-	if (!vol || !pathname) {
-		errno = EINVAL;
-		return NULL;
-	}
-
-	if (parent) {
-		ni = parent;
-	} else {
-		ni = ntfs_inode_open (vol, FILE_root);
-		if (!ni) {
-			ecrit_rapport("Couldn't open the inode of the root directory.\n");
-			goto close;
-		}
-	}
-
-	unicode = calloc (1, MAX_PATH);
-	ascii   = strdup (pathname);		/* Work with a r/w copy */
-	if (!unicode || !ascii) {
-		ecrit_rapport("Out of memory.\n");
-		goto close;
-	}
-
-	p = ascii;
-	while (p && *p && *p == PATH_SEP)	/* Remove leading /'s */
-		p++;
-	while (p && *p) {
-		q = strchr (p, PATH_SEP);	/* Find the end of the first token */
-		if (q != NULL) {
-			*q = '\0';
-			q++;
-		}
-
-		len = ntfs_mbstoucs (p, &unicode, MAX_PATH);
-		if (len < 0) {
-			ecrit_rapport("Couldn't convert name to Unicode: %s.\n", p);
-			goto close;
-		}
-
-		inum = ntfs_inode_lookup_by_name (ni, unicode, len);
-		if (inum == (u64)-1) {
-			ecrit_rapport("Couldn't find name '%s' in pathname '%s'.\n", p, pathname);
-			goto close;
-		}
-
-		if (ni != parent)
-			ntfs_inode_close (ni);
-
-		ni = ntfs_inode_open (vol, inum);
-		if (!ni) {
-			ecrit_rapport("Cannot open inode %lld: %s.\n", inum, p);
-			goto close;
-		}
-
-		p = q;
-		while (p && *p && *p == PATH_SEP)
-			p++;
-	}
-
-	result = ni;
-	ni = NULL;
-close:
-	if (ni && (ni != parent))
-		ntfs_inode_close (ni);
-	free (ascii);
-	free (unicode);
-	return result;
-}
-
 static t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int cluster)
 {
   ntfs_inode *inode;
@@ -350,9 +263,9 @@ static t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partitio
   ls->dir_list=NULL;
   ls->current_file=NULL;
 
-  inode = utils_pathname_to_inode (ls->vol, NULL, dir_data->current_directory);
+  inode = ntfs_inode_open (ls->vol, cluster);
   if (!inode) {
-    ecrit_rapport("ntfs_dir: utils_pathname_to_inode failed\n");
+    ecrit_rapport("ntfs_dir: ntfs_inode_open failed\n");
     return NULL;
   }
 
@@ -362,7 +275,7 @@ static t_file_data *ntfs_dir(t_param_disk *disk_car, const t_partition *partitio
    */
   pos = 0;
   if (inode->mrec->flags & MFT_RECORD_IS_DIRECTORY) {
-    if(ntfs_readdir(inode, &pos, ls, (ntfs_filldir_t)list_entry)<0)
+    if(ntfs_readdir(inode, &pos, ls, (ntfs_filldir_t)ntfs_list_entry)<0)
     {
       ecrit_rapport("ntfs_readdir failed\n");
     }
@@ -470,9 +383,9 @@ static int ntfs_copy(t_param_disk *disk_car, const t_partition *partition, t_dir
 {
   ntfs_inode *inode;
   struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
-  inode = utils_pathname_to_inode (ls->vol, NULL, dir_data->current_directory);
+  inode = ntfs_inode_open (ls->vol, first_inode);
   if (!inode) {
-    ecrit_rapport("ntfs_copy: utils_pathname_to_inode failed\n");
+    ecrit_rapport("ntfs_copy: ntfs_inode_open failed\n");
     return 1;
   }
   {
@@ -500,17 +413,17 @@ static int ntfs_copy(t_param_disk *disk_car, const t_partition *partition, t_dir
       block_size = index_get_size(inode);
     else
       block_size = 0;
-    new_file=MALLOC(strlen(dir_data->current_directory)+2);
-    new_file[0]='.';
-    strcpy(&new_file[1],dir_data->current_directory);
+    new_file=MALLOC(strlen(dir_data->local_dir)+strlen(dir_data->current_directory)+1);
+    strcpy(new_file,dir_data->local_dir);
+    strcat(new_file,dir_data->current_directory);
     f_out=create_file(new_file);
     if(!f_out)
     {
       ecrit_rapport("Can't create file %s: %s\n",new_file, strerror(errno));
-      FREE(new_file);
+      free(new_file);
       return 4;
     }
-    FREE(new_file);
+    free(new_file);
     offset = 0;
     for (;;)
     {
@@ -539,98 +452,88 @@ static int ntfs_copy(t_param_disk *disk_car, const t_partition *partition, t_dir
     }
     fclose(f_out);
     ntfs_attr_close(attr);
-    FREE(buffer);
+    free(buffer);
   }
   /* Finished with the inode; release it. */
   ntfs_inode_close(inode);
   return 0;
 }
 
-int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
+static void dir_partition_ntfs_close(t_dir_data *dir_data)
 {
-  t_my_data my_data;
-  static struct ntfs_device *dev;
+  const t_partition *partition;
+  t_param_disk *disk_car;
+  struct list_dir_struct *ls=(struct list_dir_struct*)dir_data->private_dir_data;
+  /* ntfs_umount() will invoke ntfs_device_free() for us. */
+  disk_car=ls->my_data->disk_car;
+  partition=ls->my_data->partition;
+  ntfs_umount(ls->vol, FALSE);
+  free(ls->my_data);
+  if(partition->boot_sector!=0)
+    io_redir_del_redir(disk_car,partition->part_offset);
+  free(ls);
+}
+#endif
+
+int dir_partition_ntfs_init(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const int debug)
+{
+#ifdef HAVE_LIBNTFS
+  struct ntfs_device *dev;
+  t_my_data *my_data=NULL;
   ntfs_volume *vol=NULL;
   if(partition->boot_sector!=0)
     io_redir_add_redir(disk_car,partition->part_offset,DEFAULT_SECTOR_SIZE,partition->part_offset+(uint64_t)partition->boot_sector*disk_car->sector_size,NULL);
-  aff_buffer(BUFFER_RESET,"Q");
-  aff_copy(window);
-  wmove(window,4,0);
-  aff_part(window,AFF_PART_ORDER,disk_car,partition);
-  my_data.partition=partition;
-  my_data.disk_car=disk_car;
-  my_data.offset=0;
   dev = ntfs_device_alloc("/", 0, &ntfs_device_testdisk_io_ops, NULL);
   if (dev)
   {
-    dev->d_private=&my_data;
+    my_data=MALLOC(sizeof(*my_data));
+    my_data->partition=partition;
+    my_data->disk_car=disk_car;
+    my_data->offset=0;
+    dev->d_private=my_data;
     /* Call ntfs_device_mount() to do the actual mount. */
     vol = ntfs_device_mount(dev, MS_RDONLY);
   }
   if (!vol) {
-    aff_part_rapport(disk_car,partition);
-    aff_buffer(BUFFER_ADD,"Can't open NTFS filesystem: %s\n",strerror (errno));
-    screen_buffer_display(window,"",NULL);
     if(partition->boot_sector!=0)
       io_redir_del_redir(disk_car,partition->part_offset);
+    free(my_data);
     ntfs_device_free(dev);
-    return 2;
+    return -1;
   }
   if (vol->flags & VOLUME_IS_DIRTY) {
     ecrit_rapport("Volume is dirty.\n");
   }
   {
-    struct list_dir_struct ls;
-    t_dir_data dir_data;
-    ls.dir_list=NULL;
-    ls.current_file=NULL;
-    ls.vol=vol;
-    dir_data.window=window;
-    dir_data.debug=debug;
-    dir_data.private_dir_data=&ls;
-    dir_data.get_dir=ntfs_dir;
-    dir_data.copy_file=ntfs_copy;
-    strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
-    dir_partition(disk_car,partition,&dir_data,FILE_root,current_cmd);
+    struct list_dir_struct *ls=(struct list_dir_struct *)MALLOC(sizeof(*ls));
+    ls->dir_list=NULL;
+    ls->current_file=NULL;
+    ls->vol=vol;
+    ls->my_data=my_data;
+    strncpy(dir_data->current_directory,"/",sizeof(dir_data->current_directory));
+    dir_data->current_inode=FILE_root;
+    dir_data->debug=debug;
+    dir_data->get_dir=ntfs_dir;
+    dir_data->copy_file=ntfs_copy;
+    dir_data->close=&dir_partition_ntfs_close;
+    dir_data->local_dir=NULL;
+    dir_data->private_dir_data=ls;
   }
-  /* ntfs_umount() will invoke ntfs_device_free() for us. */
-  ntfs_umount(vol, FALSE);
-  if(partition->boot_sector!=0)
-    io_redir_del_redir(disk_car,partition->part_offset);
   return 0;
+#else
+  return -2;
+#endif
 }
 
 const char*td_ntfs_version(void)
 {
+#ifdef HAVE_LIBNTFS
 #ifdef HAVE_NTFS_LIBNTFS_VERSION
   return ntfs_libntfs_version();
 #else
   return "avaible";
 #endif
-}
 #else
-#include "types.h"
-#include "common.h"
-#include "intrf.h"
-#include "ntfs_dir.h"
-
-int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
-{
-  aff_buffer(BUFFER_RESET,"Q");
-  aff_copy(window);
-  wmove(window,4,0);
-  aff_part(window,AFF_PART_ORDER,disk_car,partition);
-  aff_part_rapport(disk_car,partition);
-  aff_buffer(BUFFER_ADD,"Recompile with ntfsprogs library");
-  if(*current_cmd!=NULL)
-    screen_buffer_to_log();
-  else
-    screen_buffer_display(window,"",NULL);
-  return 0;
-}
-
-const char*td_ntfs_version(void)
-{
   return "none";
-}
 #endif
+}
