@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -280,7 +280,7 @@ static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_p
     }
     return new_list_part;
   }
-  FREE(new_partition);
+  free(new_partition);
   return list_part;
 }
 
@@ -316,7 +316,7 @@ static int test_structure_sun(t_list_part *list_part)
   for(element=new_list_part;element!=NULL;element=new_element)
   {
     new_element=element->next;
-    FREE(element);
+    free(element);
   }
   return res;
 }
@@ -374,7 +374,7 @@ static void init_structure_sun(const t_param_disk *disk_car,t_list_part *list_pa
     for(element=new_list_part;element!=NULL;element=new_element)
     {
       new_element=element->next;
-      FREE(element);
+      free(element);
     }
   }
 #ifdef DEBUG
