@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -118,7 +118,7 @@ t_list_part *read_part_xbox(t_param_disk *disk_car, const int debug, const int s
     return new_list_part;
   {
     struct xbox_partition *xboxlabel=(struct xbox_partition*)&buffer;
-    if (strncmp(xboxlabel->magic,"BRFR",4))
+    if (memcmp(xboxlabel->magic,"BRFR",4))
     {
       aff_buffer(BUFFER_ADD,"\nBad XBOX partition, invalid signature\n");
       return NULL;
@@ -210,7 +210,7 @@ static t_list_part *add_partition_xbox(t_param_disk *disk_car,t_list_part *list_
     }
     return new_list_part;
   }
-  FREE(new_partition);
+  free(new_partition);
   return list_part;
 }
 
@@ -246,7 +246,7 @@ static int test_structure_xbox(t_list_part *list_part)
   for(element=new_list_part;element!=NULL;element=new_element)
   {
     new_element=element->next;
-    FREE(element);
+    free(element);
   }
   return res;
 }
@@ -304,7 +304,7 @@ static void init_structure_xbox(const t_param_disk *disk_car,t_list_part *list_p
     for(element=new_list_part;element!=NULL;element=new_element)
     {
       new_element=element->next;
-      FREE(element);
+      free(element);
     }
   }
 #ifdef DEBUG
