@@ -2,7 +2,7 @@
 
     File: rfs_dir.h
 
-    Copyright (C) 2005  Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2005-2006  Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,10 +15,10 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
-int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd);
+int dir_partition_reiser_init(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const int debug);
 const char*td_reiserfs_version(void);
