@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifdef HAVE_CONFIG_H
@@ -35,6 +35,7 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#include <stdio.h>
 #include <errno.h>
 #include "types.h"
 #include "common.h"
@@ -43,8 +44,6 @@
 #include "savehdr.h"
 #define BACKUP_MAXSIZE 5120
 
-static t_backup_disk *insert_backup(t_backup_disk *list,t_backup_disk *element2insert);
-
 int save_header(t_param_disk *disk_car,t_partition *partition, const int debug)
 {
   unsigned char *buffer;
@@ -86,35 +85,22 @@ int save_header(t_param_disk *disk_car,t_partition *partition, const int debug)
   if(res>=0 && fwrite(buffer,DEFAULT_SECTOR_SIZE,256,f_backup)!=256)
     res=-1;
   fclose(f_backup);
-  FREE(buffer);
+  free(buffer);
   return res;
 }
 
-static t_backup_disk *insert_backup(t_backup_disk *list,t_backup_disk *element2insert)
-{
-  if(element2insert==NULL)
-    return list;
-  {
-    t_backup_disk *prev;
-    for(prev=list;prev!=NULL && prev->next!=NULL;prev=prev->next);
-    if(prev!=NULL)
-    {
-      prev->next=element2insert;
-    }
-    element2insert->prev=prev;
-    element2insert->next=NULL;
-  }
-  return (list!=NULL?list:element2insert);
-}
-
 t_backup_disk *partition_load(const t_param_disk *disk_car, const int debug)
 {
   FILE *f_backup;
   char *buffer;
   char *pos=NULL;
   int taille;
-  t_backup_disk *list_backup=NULL;
   t_backup_disk *new_backup=NULL;
+  t_backup_disk *list_backup;
+  list_backup=(t_backup_disk*)MALLOC(sizeof(*list_backup));
+  list_backup->list.prev= &list_backup->list;
+  list_backup->list.next = &list_backup->list;
+
   if(debug>1)
   {
     ecrit_rapport("partition_load\n");
@@ -149,7 +135,8 @@ t_backup_disk *partition_load(const t_param_disk *disk_car, const int debug)
       {
 	ecrit_rapport("new disk: %s\n",pos);
       }
-      list_backup=insert_backup(list_backup,new_backup);
+      if(new_backup!=NULL)
+	list_add_tail(&new_backup->list,&list_backup->list);
       new_backup=(t_backup_disk*)MALLOC(sizeof(*new_backup));
       new_backup->description[0]='\0';
       new_backup->list_part=NULL;
@@ -190,13 +177,13 @@ t_backup_disk *partition_load(const t_param_disk *disk_car, const int debug)
 	}
 	else
 	{
-	  FREE(new_partition);
+	  free(new_partition);
 	}
       }
       else
       {
 	ecrit_rapport("partition_load: sscanf failed\n");
-	FREE(new_partition);
+	free(new_partition);
 	pos=NULL;
       }
     }
@@ -207,9 +194,10 @@ t_backup_disk *partition_load(const t_param_disk *disk_car, const int debug)
       pos++;
     }
   }
-  list_backup=insert_backup(list_backup,new_backup);
+  if(new_backup!=NULL)
+    list_add_tail(&new_backup->list,&list_backup->list);
   fclose(f_backup);
-  FREE(buffer);
+  free(buffer);
   return list_backup;
 }
 
