@@ -2,7 +2,7 @@
 
     File: sun.c
 
-    Copyright (C) 2004-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -24,6 +24,9 @@
 #include <config.h>
 #endif
  
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -41,13 +44,20 @@ static int test_sun_i386(const t_param_disk *disk_car, const sun_partition_i386
 
 int check_sun_i386(t_param_disk *disk_car,t_partition *partition,const int debug)
 {
-  unsigned char buffer[DEFAULT_SECTOR_SIZE];
-  sun_partition_i386 *sunlabel=(sun_partition_i386*)&buffer;
-  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+0x200)!=0)
-  { return 1; }
-  if(test_sun_i386(disk_car,sunlabel,partition,debug,0)!=0)
+  unsigned char *buffer=(unsigned char*)MALLOC(SUN_PARTITION_I386_SIZE);
+  sun_partition_i386 *sunlabel=(sun_partition_i386*)buffer;
+  if(disk_car->read(disk_car,SUN_PARTITION_I386_SIZE, buffer, partition->part_offset+0x200)!=0)
+  {
+    free(buffer);
     return 1;
+  }
+  if(test_sun_i386(disk_car,sunlabel,partition,debug,0)!=0)
+  {
+    free(buffer);
+    return 1;
+  }
   set_sun_info_i386(disk_car,sunlabel,partition,debug,0);
+  free(buffer);
   return 0;
 }
 
@@ -78,7 +88,7 @@ static int test_sun_i386(const t_param_disk *disk_car, const sun_partition_i386
 	  aff_part_rapport(disk_car,new_partition);
 	}
       }
-      FREE(new_partition);
+      free(new_partition);
     }
     return 0;
   }
