@@ -2,7 +2,7 @@
 
     File: sun.h
 
-    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2006 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
@@ -48,6 +48,8 @@ typedef struct {
   uint16_t csum;       /* Label xor'd checksum */
 } sun_partition;
 
+#define SUN_PARTITION_I386_SIZE 512
+
 struct struct_sun_partition_i386 {
   unsigned char bootinfo[12];
   uint32_t magic_start;      /* Magic number */
