@@ -17,14 +17,19 @@
  * other software, or any other product whatsoever.
  *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
+ * with this program; if not, write the Free Software Foundation, Inc., 51
+ * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
+ /*
 #ifdef __GNUC__
 #else
 #error I want gcc!
 #endif
+ */
+
+#define SYSV4_SUPERBLOCK_SIZE 512
+
 #define __packed2__  __attribute__ ((packed, aligned(2)))
 
 /* inode numbers are 16 bit */
@@ -201,5 +206,5 @@ struct coh_super_block {
 };
 
 int check_sysv(t_param_disk *disk_car,t_partition *partition,const int debug);
-int recover_sysv(t_param_disk *disk_car, const unsigned char *ubh, t_partition *partition,const int debug, const int dump_ind);
+int recover_sysv(t_param_disk *disk_car, const struct sysv4_super_block *sbd, t_partition *partition,const int debug, const int dump_ind);
 
