@@ -15,8 +15,8 @@
     GNU General Public License for more details.
   
     You should have received a copy of the GNU General Public License along
-    with this program; if not, write the Free Software Foundation, Inc., 59
-    Temple Place - Suite 330, Boston MA 02111-1307, USA.
+    with this program; if not, write the Free Software Foundation, Inc., 51
+    Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 #ifndef _TESTDISK_TYPES_H
@@ -28,7 +28,10 @@
   #include <stdint.h>
   #endif
 #endif
+#ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
+#endif
+  /*
 #ifdef HAVE_NCURSES_H
 #include <ncurses.h>
 #elif defined(HAVE_NCURSES_NCURSES_H)
@@ -38,4 +41,5 @@
 #elif defined(HAVE_CURSES_H)
 #include <curses.h>
 #endif
+  */
 #endif
