@@ -17,11 +17,13 @@
  * other software, or any other product whatsoever.
  *
  * You should have received a copy of the GNU General Public License along
- * with this program; if not, write the Free Software Foundation, Inc., 59
- * Temple Place - Suite 330, Boston MA 02111-1307, USA.
+ * with this program; if not, write the Free Software Foundation, Inc., 51
+ * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  *
  */
 
+/* ufs superblock size is 1377 but there is room for 8192 */
+#define UFS_SUPERBLOCK_SIZE 2048
 
 #define UFS_BBLOCK 0
 #define UFS_BBSIZE 8192
